#ifdef GUI

#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <sim.h>
#include <param.h>
#include <geom.h>
#include <environ.h>

#include "support.h"
#include "callbacks.h"


#ifdef OPENGL
#include <gtkgl/gtkglarea.h>
#include <GL/gl.h>
#include "world3d.h"
#else
#include "world.h"
#endif

#ifdef WIN32
HANDLE simAreaUpdate_S;
HANDLE simArea_S;
#else
sem_t simAreaUpdate_S;
sem_t simArea_S;
#endif

int gui_stepping=0;

guint contextId;
guint lastMsg=0;
guint currMsg=0;
guint contextId2;
guint lastMsg2=0;
guint currMsg2=0;

gint activeRobot=-1;
extern Environ *world;
#define gui_print(msg) { currMsg = gtk_statusbar_push(GTK_STATUSBAR(statusbar1),contextId,msg); gtk_statusbar_remove(GTK_STATUSBAR(statusbar1),contextId,lastMsg); lastMsg=currMsg;} 
#define gui_print2(msg) { currMsg2 = gtk_statusbar_push(GTK_STATUSBAR(statusbar2),contextId2,msg); gtk_statusbar_remove(GTK_STATUSBAR(statusbar2),contextId2,lastMsg2); lastMsg2=currMsg2;} 

gushort pal[6][3]={{0xffff,0xffff,0xffff},
		  {0x0,0x0,0x0},
		  {0x0,0x0,0xffff},
		  {0xffff,0x0,0x0},
		  {0x0,0xffff,0x0},
		  {0xffff,0xffff,0x0}};

/* Window */
GtkWidget *window;
GtkWidget *vbox_window;
GtkWidget *menubar_window;
GtkWidget *file_menubar;
GtkWidget *file_menu;
GtkAccelGroup *file_menu_accels;
GtkWidget *quit_menu_item;
GtkWidget *table_vbox;
GtkWidget *sim_frame_table;
GtkWidget *vbox_sim_frame;
GtkWidget *toolbar_vbox_sim_frame;
GtkWidget *tmp_toolbar_icon;
GtkWidget *run_btn;
GtkWidget *paus_btn;
GtkWidget *step_btn;
GtkWidget *evolution_tgl;
GtkWidget *snapshot_btn;
GtkWidget *simulatorArea;
GtkWidget *draw_frame_table;
GtkWidget *vbox_draw_frame;
GtkWidget *robot_frame_vbox;
GtkWidget *vbox_robot_frame;
GtkWidget *toolbar8;
GtkWidget *wheel_rbtn;
GtkWidget *arrow_rbtn;
GtkWidget *toolbar10;
GtkWidget *body_tgl;
GtkWidget *trace_tgl;
GtkWidget *Robots_frm;
GtkWidget *draw_tgl;
GtkWidget *generation_progress_bar;
GtkWidget *statusbar1;
GtkWidget *statusbar2;
GtkAdjustment *adj;
/* end window */

GtkWidget **simRobots;
GtkWidget *window1;
GtkWidget *simArea;      /* simArea widget     */
#ifndef OPENGL
GdkPixmap *bb_simArea;   /* simArea backbuffer */
GdkPixmap *bg_simArea;   /* simArea background */
#endif
GdkPixmap **bb_simRobots;
GdkColormap *cm_simArea; /* simArea colormap   */
GdkColor **pal_simArea;  /* simArea colors     */
#ifdef WIN32
const gchar defaultFontName[]="curior";
#else
const gchar defaultFontName[]="-*-helvetica-bold-r-normal--*-120-*-*-*-*-iso8859-1";
#endif
GdkFont *defaultFont;
GdkGC *pen_simArea;
GdkGC **internal_blit_simRobots;
#ifndef OPENGL
GdkGC *blit_simArea;
GdkGC *internal_blit_simArea;
#endif
gint colors = 256;
gint definedColors = 6;
guint simAreaTimer;
/* Simulator area size, robotbody and robottype*/
int sim_area_x_size=200;
int sim_area_y_size=200;
BOOL_T robotBody = BTRUE;
BOT_T robotType = ARROW;

int sensorAreaVisible=0;
int running = 0;
/* Prototypes */
gint init_pal(gpointer data);
gint on_progress_update(gpointer);
gint on_gui_update(gpointer);
void on_expose_event(GtkWidget *w, gpointer user_data);
void on_quit_req(GtkWidget *w, gpointer user_data);
void gui_change_active_robot(GtkWidget *w,GdkEventButton *event);
GtkWidget * create_main_window();

void gui_setWorldSize(int x, int y){
  sim_area_x_size = x;
  sim_area_y_size = y;
}

void gui_init(){
  int i;
  GtkWidget *rob_frm;
  GtkWidget *tmp;
  GtkWidget *v1,*v2;
  GtkWidget **distribute;
  gchar str[6];
#ifdef WIN32
  simAreaUpdate_S = CreateSemaphore(NULL,0,1,NULL);
  simArea_S = CreateSemaphore(NULL,0,1,NULL);
#else
  sem_init(&simAreaUpdate_S,0,0);
  sem_init(&simArea_S,0,0);
#endif

  notTrajactoryRobot = 1;
  
  /*
    init simulator area
  */
  window1 = create_main_window();
  gtk_widget_show(window1);
  
  contextId = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar1),"User messages");
  lastMsg = gtk_statusbar_push(GTK_STATUSBAR(statusbar1),contextId,"User Interface Started");
 
  contextId2 = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar2),"User messages");
  lastMsg2 = gtk_statusbar_push(GTK_STATUSBAR(statusbar2),contextId2,"User Interface Started");

  simArea = lookup_widget(window1,"simulatorArea");
  gtk_signal_connect(GTK_OBJECT(simArea),"expose_event",
		     GTK_SIGNAL_FUNC(on_expose_event),
		     NULL);
#ifndef OPENGL
  bb_simArea = gdk_pixmap_new(simArea->window,
  			      sim_area_x_size,
  			      sim_area_y_size,
			      -1);

  bb_simArea = gdk_pixmap_ref(bb_simArea); 
  
  bg_simArea = gdk_pixmap_new(simArea->window,
  			      sim_area_x_size,
  			      sim_area_y_size,
			      -1);

  bg_simArea = gdk_pixmap_ref(bg_simArea); 
#endif
  /* Font */
  defaultFont = gdk_font_load(defaultFontName);
  defaultFont = gdk_font_ref(defaultFont);
  /* Robot windows */
  rob_frm = lookup_widget(window1,"Robots_frm");

  tmp = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (tmp);
  gtk_object_set_data_full (GTK_OBJECT (window1), "rob_hbox", tmp,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tmp);
  gtk_container_add (GTK_CONTAINER (rob_frm), tmp);

  v1 = gtk_toolbar_new(GTK_ORIENTATION_VERTICAL,GTK_TOOLBAR_ICONS);
  gtk_widget_ref(v1);
  gtk_object_set_data_full(GTK_OBJECT (window1), "v1", v1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (v1);
  gtk_box_pack_start (GTK_BOX (tmp), v1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (v1), 1);

  v2 = gtk_toolbar_new(GTK_ORIENTATION_VERTICAL,GTK_TOOLBAR_ICONS);
  gtk_widget_ref(v2);
  gtk_object_set_data_full (GTK_OBJECT (window1), "v2", v2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (v2);
  gtk_box_pack_start (GTK_BOX (tmp), v2, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (v2), 1);
  simRobots = (GtkWidget**) malloc(sizeof(GtkWidget*)*paramNrOfRobots);
  distribute = (GtkWidget**) malloc(sizeof(GtkWidget*)*2);
  distribute[0] = v1;
  distribute[1] = v2;
  bb_simRobots = (GdkPixmap**) malloc(sizeof(GdkPixmap*)*paramNrOfRobots);
  internal_blit_simRobots = (GdkGC**) malloc(sizeof(GdkGC*)*paramNrOfRobots);
  for(i=0; i < paramNrOfRobots; i++){
    bb_simRobots[i] = gdk_pixmap_new(NULL,51,90,gdk_visual_get_best_depth());
    bb_simRobots[i] = gdk_pixmap_ref(bb_simRobots[i]);
    internal_blit_simRobots[i] = gdk_gc_new((GdkWindow*)bb_simRobots[i]);
    internal_blit_simRobots[i] = gdk_gc_ref(internal_blit_simRobots[i]);

    simRobots[i] = gtk_drawing_area_new();
    gtk_signal_connect(GTK_OBJECT(simRobots[i]),"button_press_event",
				  GTK_SIGNAL_FUNC(gui_change_active_robot),
				  NULL);
    gtk_widget_set_events (simRobots[i],
			   GDK_BUTTON_PRESS_MASK|GDK_EXPOSURE_MASK);
    gtk_widget_ref(simRobots[i]);
    gtk_drawing_area_size(GTK_DRAWING_AREA(simRobots[i]),51,90);

    sprintf(str,"rob%d",i);
    gtk_widget_set_name(simRobots[i],str);
    gtk_object_set_data_full(GTK_OBJECT(window1),str,simRobots[i],
			     (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show(simRobots[i]);
    gtk_toolbar_append_element(GTK_TOOLBAR(distribute[i%2]), 
			       GTK_TOOLBAR_CHILD_WIDGET,
			       simRobots[i], 
			       str,
			       NULL,NULL,NULL,NULL,NULL);
  }
  free(distribute);
  colors = 256;
  cm_simArea = gdk_window_get_colormap(simArea->window);
  
  pal_simArea = (GdkColor**) malloc(sizeof(GdkColor*)*colors);
  for(i=0; i < colors; i++){
    pal_simArea[i] = (GdkColor*)malloc(sizeof(GdkColor));
    if(i < 6){
      pal_simArea[i]->red = pal[i][0];
      pal_simArea[i]->green = pal[i][1];
      pal_simArea[i]->blue = pal[i][2];
    }
    else{
      pal_simArea[i]->red = int(65536.0 * cos(i/double(63)*M_PI/3));
      pal_simArea[i]->green = int(65536.0 * sin(i/double(63)*M_PI/3));
      pal_simArea[i]->blue = 0;
    }
    gdk_colormap_alloc_color(cm_simArea,
  			     pal_simArea[i],
  			     1, /* writeable ->  private */
  			     1);/* bestmatch */
  }
#ifndef OPENGL
  pen_simArea = gdk_gc_new((GdkWindow *)bb_simArea);
  pen_simArea = gdk_gc_ref(pen_simArea);
  blit_simArea = gdk_gc_new((GdkWindow*)bb_simArea);
  blit_simArea = gdk_gc_ref(blit_simArea);
  internal_blit_simArea = gdk_gc_new((GdkWindow*)bb_simArea);
  internal_blit_simArea = gdk_gc_ref(internal_blit_simArea);
#endif
  guint prog_up = gtk_timeout_add(500,
		  on_progress_update,
		  NULL);
  guint gui_up = gtk_timeout_add(100,
		  on_gui_update,
		  NULL);
#ifdef WIN32
  ReleaseSemaphore(gui_ready,1,NULL);
  gdk_threads_enter();
  running = 1;
  while(running && gtk_main_iteration_do(TRUE));
  //while (gtk_events_pending()&& gtk_main_iteration_do(TRUE)) ;
  //gtk_main();
  
	gtk_timeout_remove(prog_up);
    gtk_timeout_remove(gui_up);


  gdk_threads_leave();
#else
  sem_post(&gui_ready);

  gdk_threads_enter();
  gtk_main();
  gdk_threads_leave();
 
#endif

  
  sim_shutdown();
  pthread_exit(0);
}

gint init_pal(gpointer data){
  
  return (FALSE);
}

void on_quit_req(GtkWidget *w, gpointer user_data){
	running = 0;
}

gint on_progress_update(gpointer data){
  gfloat val;
  static GtkWidget *pbar=0;

  if(!pbar)
    pbar = lookup_widget(window1,"generation_progress_bar");
  
  if(startGeneration==-1)
    val = (gfloat)(100.0*((gfloat)(gen)/ (gfloat)(nrOfGenerations)));
  else
    val = (gfloat)(100.0*((gfloat)(gen-startGeneration)/
			  (gfloat)(nrOfGenerations-startGeneration)));
  if(val < 1)
    val = 1;
  if(val > 100)
    val = 100;
  gdk_threads_enter();
  gtk_progress_set_value(GTK_PROGRESS(pbar),val);
  gdk_threads_leave();

  return TRUE;

}

void on_expose_event(GtkWidget *w, gpointer user_data){
  for(int i=0; i < paramNrOfRobots; i++){
    gdk_draw_pixmap(simRobots[i]->window,
		    internal_blit_simRobots[i],
		    bb_simRobots[i],
		    0,0,0,0,
		    -1,-1);
  }
#ifdef OPENGL
	drawopengl(w,(GdkEventExpose *)user_data);
#else
  gdk_draw_pixmap(simArea->window,
		  internal_blit_simArea,
		  bb_simArea,
		  0,0,0,0,
		  -1,-1);
#endif
#ifdef WIN32
  ReleaseSemaphore(simAreaUpdate_S,1,NULL);
#else
  sem_post(&simAreaUpdate_S);
#endif
}

gint on_gui_update(gpointer data){
#ifdef WIN32
   if(WaitForSingleObject(simArea_S,10)==WAIT_OBJECT_0){
#else
  if(sem_trywait(&simArea_S) == 0){
#endif
    gdk_threads_enter();
    gtk_widget_draw(simArea,NULL);
    gdk_threads_leave();
  }
  return TRUE;
}


void
on_button_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
   printf("Klicked button\n");
}


void
on_run_btn_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  gui_stepping=0;
#ifdef WIN32
  ReleaseSemaphore(gui_run,1,NULL);
  ReleaseSemaphore(gui_step,1,NULL);
#else
  sem_post(&gui_run);
  sem_post(&gui_step);
#endif
  gui_print("Running simulation");
#ifdef DEBUG_GUI
  printf("run\n");
#endif
}


void
on_paus_btn_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gui_stepping=1;
  gui_print("Paus...");
#ifdef DEBUG_GUI
  printf("paus\n");
#endif

}


void
on_step_btn_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
#ifdef WIN32
  ReleaseSemaphore(gui_step,1,NULL);
#else
 sem_post(&gui_step);
#endif
 gui_print("Stepping 100 ms");
#ifdef DEBUG_GUI
  printf("step\n");
#endif

}


void
on_evolution_tgl_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

}


void
on_wheel_rbtn_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  robotType = WHEEL;
  gui_print("Drawing wheel axis robot");
#ifdef DEBUG_GUI
  printf("wheel robot\n");
#endif

}


void
on_arrow_rbtn_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  robotType = ARROW;
  gui_print("Drawing arrow robot");
#ifdef DEBUG_GUI
  printf("arrow robot\n");
#endif

}


void
on_body_tgl_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{  
  switch(robotBody){
  case BFALSE: robotBody = BTRUE; gui_print("Robot body on"); break;
  case BTRUE: robotBody = BFALSE; gui_print("Robot body off"); break;
  }
#ifdef DEBUG_GUI
  printf("robotBody %d\n",robotBody);
#endif
}


void
on_trace_tgl_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if(notTrajactoryRobot){
    notTrajactoryRobot=0;
    gui_print("Trajactory trace turned on");
  }
  else{
    gui_print("Trajactory turned turned off");
    notTrajactoryRobot=1;
  }
}


void
on_refresh_btn_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
}
void
gui_change_active_robot(GtkWidget *w, GdkEventButton *event){
  int whichRobot=-1;
  switch(event->button){
  case 1:
  case 2:
  case 3:
    sscanf(gtk_widget_get_name(w),"rob%d",&whichRobot);
    if(whichRobot> -1 && whichRobot < paramNrOfRobots)
      activeRobot = whichRobot;
    break;
  default:
    break;
  }
}

#ifndef OPENGL
void gui_move_active_robot(GtkWidget *w, GdkEventButton *event){
  switch(event->button){
  case 1:
    if(world){
      if(activeRobot>-1){
	//	printf("Active robot %d, new (x,y)(%f,%f)\n",activeRobot,event->x / xScale,event->y / xScale);
	world->robot[activeRobot]->setX(event->x / xScale);
	world->robot[activeRobot]->setY(event->y / xScale);
      }
    }
    break;
  default:
    break;
  }
}
#endif
void
on_draw_tgl_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  switch(gfxUpdate){
  case 1: gfxUpdate=0; gui_print("GUI update turned off"); break;
  case 0: gfxUpdate=1; gui_print("GUI update turned on");break;
  }
}


GtkWidget* create_main_window()
{ /* main window */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (window), "window", window);
  gtk_window_set_title (GTK_WINDOW (window), _("YAKS - Yet Another Khepera Simulator"));
  gtk_signal_connect(GTK_OBJECT(window),"destroy",
		     (GtkSignalFunc)/*gtk_main_quit*/on_quit_req,
		     NULL);
     
  { /* vbox window */
    vbox_window = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox_window);
    gtk_object_set_data_full (GTK_OBJECT (window), "vbox_window", vbox_window,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox_window);
    gtk_container_add (GTK_CONTAINER (window), vbox_window);
    { /* vbox ->  menu_bar */
      menubar_window = gtk_menu_bar_new ();
      gtk_widget_ref (menubar_window);
      gtk_object_set_data_full (GTK_OBJECT (window), "menubar_window", menubar_window,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (menubar_window);
      gtk_box_pack_start (GTK_BOX (vbox_window), menubar_window, FALSE, FALSE, 0);
      
      file_menubar = gtk_menu_item_new_with_label (_("File"));
      gtk_widget_ref (file_menubar);
      gtk_object_set_data_full (GTK_OBJECT (window), "file_menubar", file_menubar,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (file_menubar);
      gtk_container_add (GTK_CONTAINER (menubar_window), file_menubar);
      
      file_menu = gtk_menu_new ();
      gtk_widget_ref (file_menu);
      gtk_object_set_data_full (GTK_OBJECT (window), "file_menu", file_menu,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_menu_item_set_submenu (GTK_MENU_ITEM (file_menubar), file_menu);
      file_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (file_menu));
      
      quit_menu_item = gtk_menu_item_new_with_label (_("Quit"));
      gtk_widget_ref (quit_menu_item);
      gtk_object_set_data_full (GTK_OBJECT (window), "quit_menu_item", quit_menu_item,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (quit_menu_item);
      gtk_container_add (GTK_CONTAINER (file_menu), quit_menu_item);
    }

    { /* vbox -> table       */
      table_vbox = gtk_table_new (1, 2, FALSE);
      gtk_widget_ref (table_vbox);
      gtk_object_set_data_full (GTK_OBJECT (window), "table_vbox", table_vbox,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (table_vbox);
      gtk_box_pack_start (GTK_BOX (vbox_window), table_vbox, TRUE, TRUE, 0);
      { /* simulation frame */
	sim_frame_table = gtk_frame_new (_("Simulation"));
	gtk_widget_ref (sim_frame_table);
	gtk_object_set_data_full (GTK_OBJECT (window), "sim_frame_table", sim_frame_table,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (sim_frame_table);
	gtk_table_attach (GTK_TABLE (table_vbox), sim_frame_table, 0, 1, 0, 1,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (sim_frame_table), 3);
	gtk_frame_set_label_align (GTK_FRAME (sim_frame_table), 0.5, 0.5);
	gtk_frame_set_shadow_type (GTK_FRAME (sim_frame_table), GTK_SHADOW_ETCHED_OUT);
	{ /* simulation vbox */

	  vbox_sim_frame = gtk_vbox_new (FALSE, 0);
	  gtk_widget_ref (vbox_sim_frame);
	  gtk_object_set_data_full (GTK_OBJECT (window), "vbox_sim_frame", vbox_sim_frame,
				    (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (vbox_sim_frame);
	  gtk_container_add (GTK_CONTAINER (sim_frame_table), vbox_sim_frame);
	  { /* Simulation buttons -  toolbar_vbox_sim_frame */
	    toolbar_vbox_sim_frame = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	    gtk_widget_ref (toolbar_vbox_sim_frame);
	    gtk_object_set_data_full (GTK_OBJECT (window), "toolbar_vbox_sim_frame", toolbar_vbox_sim_frame,
				      (GtkDestroyNotify) gtk_widget_unref);
	    gtk_widget_show (toolbar_vbox_sim_frame);
	    gtk_box_pack_start (GTK_BOX (vbox_sim_frame), toolbar_vbox_sim_frame, FALSE, FALSE, 0);
	
	    tmp_toolbar_icon = create_pixmap (window, "pill_play.xpm");
	    gtk_widget_ref(tmp_toolbar_icon);
	    run_btn = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_vbox_sim_frame),
						  GTK_TOOLBAR_CHILD_BUTTON,
						  NULL,
						  _("Run"),
						  NULL, NULL,
						  tmp_toolbar_icon, NULL, NULL);
	    gtk_widget_ref (run_btn);
	    gtk_object_set_data_full (GTK_OBJECT (window), "run_btn", run_btn,
				      (GtkDestroyNotify) gtk_widget_unref);
	    gtk_widget_show (run_btn);
	
	    tmp_toolbar_icon = create_pixmap (window, "pill_paus.xpm");
	    gtk_widget_ref(tmp_toolbar_icon);
	    paus_btn = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_vbox_sim_frame),
						   GTK_TOOLBAR_CHILD_BUTTON,
						   NULL,
						   _("Pause"),
						   NULL, NULL,
						   tmp_toolbar_icon, NULL, NULL);
	    gtk_widget_ref (paus_btn);
	    gtk_object_set_data_full (GTK_OBJECT (window), "paus_btn", paus_btn,
				      (GtkDestroyNotify) gtk_widget_unref);
	    gtk_widget_show (paus_btn);
	
	    tmp_toolbar_icon = create_pixmap (window, "pill_step.xpm");
	    gtk_widget_ref(tmp_toolbar_icon);
	    step_btn = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_vbox_sim_frame),
						   GTK_TOOLBAR_CHILD_BUTTON,
						   NULL,
						   _("Step"),
						   NULL, NULL,
						   tmp_toolbar_icon, NULL, NULL);
	    gtk_widget_ref (step_btn);
	    gtk_object_set_data_full (GTK_OBJECT (window), "step_btn", step_btn,
				      (GtkDestroyNotify) gtk_widget_unref);
	    gtk_widget_show (step_btn);

	    tmp_toolbar_icon = create_pixmap (window, "pill_user.xpm");
	    gtk_widget_ref(tmp_toolbar_icon);
	    evolution_tgl = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_vbox_sim_frame),
							GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
							NULL,
							_("togglebutton5"),
							NULL, NULL,
							tmp_toolbar_icon, NULL, NULL);
	    gtk_widget_ref (evolution_tgl);
	    gtk_object_set_data_full (GTK_OBJECT (window), "user_tgl", evolution_tgl,
				      (GtkDestroyNotify) gtk_widget_unref);
	    gtk_widget_show (evolution_tgl);
	    tmp_toolbar_icon = create_pixmap (window, "pill_norefresh.xpm");
	    gtk_widget_ref(tmp_toolbar_icon);
	    draw_tgl = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_vbox_sim_frame),
						   GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
						   NULL,
						   _("togglebutton8"),
						   NULL, NULL,
						   tmp_toolbar_icon, NULL, NULL);
	    gtk_widget_ref (draw_tgl);
	    gtk_object_set_data_full (GTK_OBJECT (window), "draw_tgl", draw_tgl,
				      (GtkDestroyNotify) gtk_widget_unref);
	    gtk_widget_show (draw_tgl);
#ifndef OPENGL
#ifdef HAVE_IMLIB
	    tmp_toolbar_icon = create_pixmap (window, "pill_snapshot.xpm");
	    gtk_widget_ref(tmp_toolbar_icon);
	    snapshot_btn = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_vbox_sim_frame),
						       GTK_TOOLBAR_CHILD_BUTTON,
						       NULL,
						       _("Snapshot"),
						       NULL, NULL,
						       tmp_toolbar_icon, NULL, NULL);
	    gtk_widget_ref (snapshot_btn);
	    gtk_object_set_data_full (GTK_OBJECT (window), "snapshot", snapshot_btn,
				      (GtkDestroyNotify) gtk_widget_unref);
	    gtk_widget_show (snapshot_btn);
#endif
#endif

	  }
#ifdef OPENGL
	  {
			int attrlist[] = {
				GDK_GL_RGBA,
				GDK_GL_DOUBLEBUFFER,
				GDK_GL_NONE
			};
		  simulatorArea = GTK_WIDGET(gtk_gl_area_new(attrlist));
	    gtk_widget_ref(simulatorArea);
	    gtk_object_set_data_full(GTK_OBJECT(window),"simulatorArea",simulatorArea,
				     (GtkDestroyNotify) gtk_widget_unref);

	GTK_WIDGET_SET_FLAGS (simulatorArea, GTK_CAN_FOCUS);
	if ( GTK_WIDGET_CAN_FOCUS(simulatorArea) )
	{
		gtk_window_set_focus ( GTK_WINDOW (window), simulatorArea );
	}

	    gtk_signal_connect_after( GTK_OBJECT(simulatorArea), "key_press_event",
                    GTK_SIGNAL_FUNC(opengl_key_press_callback), 
		    NULL);

	    gtk_signal_connect (GTK_OBJECT(simulatorArea), "configure_event",
		    GTK_SIGNAL_FUNC(opengl_configure), NULL);

		/* set minimum size */
//		gtk_widget_set_usize(GTK_WIDGET(simulatorArea), sim_area_x_size, sim_area_y_size);
		gtk_widget_show(simulatorArea);
		gtk_box_pack_start (GTK_BOX (vbox_sim_frame), simulatorArea, TRUE, TRUE, 0);
	}
#else
	
	  { /* simulator area */
	    simulatorArea = gtk_drawing_area_new();
	    gtk_widget_set_events (simulatorArea,
			   GDK_BUTTON_PRESS_MASK|GDK_EXPOSURE_MASK);
	    gtk_signal_connect(GTK_OBJECT(simulatorArea),"button_press_event",
				  GTK_SIGNAL_FUNC(gui_move_active_robot),
				  NULL);
	    gtk_widget_set_name(simulatorArea,"simulatorArea");
	    gtk_drawing_area_size(GTK_DRAWING_AREA(simulatorArea),sim_area_x_size,sim_area_y_size);
	    gtk_widget_ref(simulatorArea);
	    gtk_object_set_data_full(GTK_OBJECT(window),"simulatorArea",simulatorArea,
				     (GtkDestroyNotify) gtk_widget_unref);
	    gtk_widget_show(simulatorArea);
	    gtk_box_pack_start (GTK_BOX (vbox_sim_frame), simulatorArea, TRUE, TRUE, 0);
	  }
#endif
	}
      }
      { /* draw_frame_table */
	draw_frame_table = gtk_frame_new (_("Drawing"));
	gtk_widget_ref (draw_frame_table);
	gtk_object_set_data_full (GTK_OBJECT (window), "draw_frame_table", draw_frame_table,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (draw_frame_table);
	gtk_table_attach (GTK_TABLE (table_vbox), draw_frame_table, 1, 2, 0, 1,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (draw_frame_table), 2);
	gtk_frame_set_label_align (GTK_FRAME (draw_frame_table), 0.5, 0.5);
	gtk_frame_set_shadow_type (GTK_FRAME (draw_frame_table), GTK_SHADOW_ETCHED_OUT);
	
	vbox_draw_frame = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox_draw_frame);
	gtk_object_set_data_full (GTK_OBJECT (window), "vbox_draw_frame", vbox_draw_frame,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox_draw_frame);
	gtk_container_add (GTK_CONTAINER (draw_frame_table), vbox_draw_frame);
	{ /*robot frame -> vbox_draw_frame -> drawing_frame */
	  robot_frame_vbox = gtk_frame_new (_("Robot"));
	  gtk_widget_ref (robot_frame_vbox);
	  gtk_object_set_data_full (GTK_OBJECT (window), "robot_frame_vbox", robot_frame_vbox,
				    (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (robot_frame_vbox);
	  gtk_box_pack_start (GTK_BOX (vbox_draw_frame), robot_frame_vbox, FALSE, TRUE, 0);
	  gtk_frame_set_shadow_type (GTK_FRAME (robot_frame_vbox), GTK_SHADOW_ETCHED_OUT);
	  
	  vbox_robot_frame = gtk_vbox_new (FALSE, 0);
	  gtk_widget_ref (vbox_robot_frame);
	  gtk_object_set_data_full (GTK_OBJECT (window), "vbox_robot_frame", vbox_robot_frame,
				  (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (vbox_robot_frame);
	  gtk_container_add (GTK_CONTAINER (robot_frame_vbox), vbox_robot_frame);
	  
	  toolbar8 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	  gtk_widget_ref (toolbar8);
	  gtk_object_set_data_full (GTK_OBJECT (window), "toolbar8", toolbar8,
				    (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (toolbar8);
	  gtk_box_pack_start (GTK_BOX (vbox_robot_frame), toolbar8, FALSE, FALSE, 0);
	  gtk_container_set_border_width (GTK_CONTAINER (toolbar8), 4);
	  
	  tmp_toolbar_icon = create_pixmap (window, "pill_wheel.xpm");
	  gtk_widget_ref(tmp_toolbar_icon);
	  wheel_rbtn = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar8),
						   GTK_TOOLBAR_CHILD_RADIOBUTTON,
						   NULL,
						   _("Wheel"),
						   NULL, NULL,
						   tmp_toolbar_icon, NULL, NULL);
	  gtk_widget_ref (wheel_rbtn);
	  gtk_object_set_data_full (GTK_OBJECT (window), "wheel_rbtn", wheel_rbtn,
				    (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (wheel_rbtn);
	  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (wheel_rbtn), FALSE);
	  
	  tmp_toolbar_icon = create_pixmap (window, "pill_arrow.xpm");
	  gtk_widget_ref(tmp_toolbar_icon);
	  arrow_rbtn = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar8),
						   GTK_TOOLBAR_CHILD_RADIOBUTTON,
						   wheel_rbtn,
						   _("Arrow\n"),
						   NULL, NULL,
						   tmp_toolbar_icon, NULL, NULL);
	  gtk_widget_ref (arrow_rbtn);
	  gtk_object_set_data_full (GTK_OBJECT (window), "arrow_rbtn", arrow_rbtn,
				    (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (arrow_rbtn);
	  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (arrow_rbtn), FALSE);
	  
	  toolbar10 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	  gtk_widget_ref (toolbar10);
	  gtk_object_set_data_full (GTK_OBJECT (window), "toolbar10", toolbar10,
				    (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (toolbar10);
	  gtk_box_pack_start (GTK_BOX (vbox_robot_frame), toolbar10, FALSE, FALSE, 0);
	  gtk_container_set_border_width (GTK_CONTAINER (toolbar10), 4);
	  
	  tmp_toolbar_icon = create_pixmap (window, "pill_bodyb.xpm");
	  gtk_widget_ref(tmp_toolbar_icon);
	  body_tgl = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar10),
						 GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
						 NULL,
						 _("Body"),
						 NULL, NULL,
						 tmp_toolbar_icon, NULL, NULL);
	  gtk_widget_ref (body_tgl);
	  gtk_object_set_data_full (GTK_OBJECT (window), "body_tgl", body_tgl,
				    (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (body_tgl);
	  
	  tmp_toolbar_icon = create_pixmap (window, "pill_traceb.xpm");
	  gtk_widget_ref(tmp_toolbar_icon);
	  trace_tgl = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar10),
						  GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
						  NULL,
						  _("togglebutton7"),
						  NULL, NULL,
						  tmp_toolbar_icon, NULL, NULL);
	  gtk_widget_ref (trace_tgl);
	  gtk_object_set_data_full (GTK_OBJECT (window), "trace_tgl", trace_tgl,
				    (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (trace_tgl);
	  
	  Robots_frm = gtk_frame_new (_("Robots"));
	  gtk_widget_ref (Robots_frm);
	  gtk_object_set_data_full (GTK_OBJECT (window), "Robots_frm", Robots_frm,
				    (GtkDestroyNotify) gtk_widget_unref);
	  gtk_widget_show (Robots_frm);
	  gtk_box_pack_start (GTK_BOX (vbox_draw_frame), Robots_frm, TRUE, TRUE, 0);
	  gtk_frame_set_shadow_type (GTK_FRAME (Robots_frm), GTK_SHADOW_ETCHED_OUT);
	}
      }
    }
    { /* generation progressbar  */      
      adj = (GtkAdjustment *) gtk_adjustment_new (0, 1, 100, 0, 0, 0);
      
      
      generation_progress_bar = gtk_progress_bar_new_with_adjustment (adj);
      
      gtk_widget_ref (generation_progress_bar);
      gtk_object_set_data_full (GTK_OBJECT (window), "generation_progress_bar", generation_progress_bar,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (generation_progress_bar);
      gtk_box_pack_start (GTK_BOX (vbox_window), generation_progress_bar, FALSE, FALSE, 0);
      gtk_progress_set_show_text (GTK_PROGRESS (generation_progress_bar), TRUE);
      gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR (generation_progress_bar),GTK_PROGRESS_CONTINUOUS);
      gtk_progress_set_activity_mode(GTK_PROGRESS (generation_progress_bar),FALSE);
      gtk_progress_set_format_string (GTK_PROGRESS (generation_progress_bar),"(%p%%)");
    }
    { /* status bar */
      statusbar1 = gtk_statusbar_new ();
      gtk_widget_ref (statusbar1);
      gtk_object_set_data_full (GTK_OBJECT (window), "statusbar1", statusbar1,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (statusbar1);
      gtk_box_pack_start (GTK_BOX (vbox_window), statusbar1, FALSE, FALSE, 0);
    }
	{ /* status bar 2 */
      statusbar2 = gtk_statusbar_new ();
      gtk_widget_ref (statusbar2);
      gtk_object_set_data_full (GTK_OBJECT (window), "statusbar2", statusbar2,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (statusbar2);
      gtk_box_pack_start (GTK_BOX (vbox_window), statusbar2, FALSE, FALSE, 0);
    }
  }
  gtk_signal_connect (GTK_OBJECT (quit_menu_item), "activate",
		      GTK_SIGNAL_FUNC (/*gtk_main_quit*/on_quit_req),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (run_btn), "clicked",
		      GTK_SIGNAL_FUNC (on_run_btn_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (paus_btn), "clicked",
		      GTK_SIGNAL_FUNC (on_paus_btn_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (step_btn), "clicked",
		      GTK_SIGNAL_FUNC (on_step_btn_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (evolution_tgl), "toggled",
		      GTK_SIGNAL_FUNC (on_evolution_tgl_toggled),
			NULL);
  gtk_signal_connect (GTK_OBJECT (wheel_rbtn), "toggled",
		      GTK_SIGNAL_FUNC (on_wheel_rbtn_toggled),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (arrow_rbtn), "toggled",
		      GTK_SIGNAL_FUNC (on_arrow_rbtn_toggled),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (body_tgl), "toggled",
		      GTK_SIGNAL_FUNC (on_body_tgl_toggled),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (trace_tgl), "toggled",
		      GTK_SIGNAL_FUNC (on_trace_tgl_toggled),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (draw_tgl), "toggled",
		      GTK_SIGNAL_FUNC (on_draw_tgl_toggled),
		      NULL);
#ifndef OPENGL
#ifdef HAVE_IMLIB
  gtk_signal_connect(GTK_OBJECT(snapshot_btn),"clicked",
		     GTK_SIGNAL_FUNC(gui_snapshot),
		     NULL);
#endif
#endif
  
  return window;
}

    
#endif