#ifdef OPENGL
#include "callbacks.h"
#include "world3d.h"
#include "sim.h"
#include "environ.h"
#include "robot.h"
#include "geom.h"
#include "param.h"
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <gtkgl/gtkglarea.h>

// WALL_SEGMENT must not be greater than 1000
#define WALL_SEGMENT 200	// 50 is nice when lights
#define WALL_WIDTH 3.0
#define WALL_HEIGHT 32.0
// 1-((WALL_SEGMENT-WALL_WIDTH*2)/WALL_SEGMENT)
//#define LOW_TEX_WIDTH 0.92
#define LOW_TEX_WIDTH (WALL_SEGMENT-WALL_WIDTH*2.0)/WALL_SEGMENT
// 1-((WALL_SEGMENT-WALL_HEIGHT)/WALL_SEGMENT)
#define LOW_TEX_HEIGHT 0.8
#define ROBOT_HEIGHT 22
#define LOW_ROBOT_TEX_HEIGHT 1.0-(46.0/256.0)
#define ZONE_HEIGHT 40
#define SOBST_HEIGHT 45
//#define LOW_SOBST_TEX_HEIGHT 1.0-(46.0/256.0)
#define LOW_SOBST_TEX_HEIGHT 114.0/256.0
#define ROBOT_TRAIL_LENGTH 100
#define ROBOT_TRAIL_DRAW_STEP 10	// draw only every 10th stored pos.

#define DARKER_WALL_SIDES

typedef struct {
   GLfloat x, y, z;
   GLfloat focusX, focusY, focusZ;
   GLfloat upX, upY, upZ;
   GLfloat thirdDist, thirdAngle, thirdHeight;
   double camFactor;		// speed that camera position is moved
   double focusFactor;		// speed that focus is moved
   bool sloppy;			// move slowly after or straight to
   int focusOnRobot;		// which robot# focus is set on
   int mode;			// which of the different view modes is in use
} Camera;

typedef struct {
   GLubyte *imageData;		// Image Data (Up To 32 Bits)
   GLuint bpp;			// Image Color Depth In Bits Per Pixel
   GLuint width;		// Image Width
   GLuint height;		// Image Height
   GLuint texID;		// Texture ID Used To Select A Texture
} TextureImage;			// Structure Name

typedef struct {
   double x, y, angle;
} RobotTrail;

Camera camera;
TextureImage textures[7];	// Storage For Textures
bool showZones = true, reflections = false;
bool lighting = false, floorLights = false;
bool showIdentities = true, showRubberBands = true;
int initialized = false, trails = false;
bool keys[256];
float xt = -500.0f, yt = -500.0f; // changed in initopengl
float bigaxis = 500.0; // stores the biggest of world->dx and world->dy (set by initopengl)
double textX = 6.0;
char *textStr = NULL;
double floorAlpha = 0.4;
GLuint fontBase;		// Base Display List For The Font
RobotTrail trail[8][ROBOT_TRAIL_LENGTH];

void initCamera(Camera * camera)
{
   camera->x = 0.0;
   camera->y = 0.0;
   camera->z = bigaxis+10;
   camera->focusX = 0.0;
   camera->focusY = 0.0;
   camera->focusZ = 0.0;
   camera->upX = 0.0;
   camera->upY = 1.0;
   camera->upZ = 0.0;
   camera->thirdDist = 100.0;
   camera->thirdAngle = 0.0;
   camera->thirdHeight = 100.0;
   camera->camFactor = 0.02;
   camera->focusFactor = 0.15;
   camera->sloppy = true;
   camera->focusOnRobot = 0;
   camera->mode = 0;
}

void showLamps()
{
   for (int i = 0; i < world->getNrOfLights() && (i + world->nrOfRobots) < 8; i++) {
      GLfloat diffuseLight[] = { 10, 10, 9, 1 };
      GLfloat positionLight[] =  { world->light[i]->x + xt, 
				   (world->dy - world->light[i]->y) + yt,
				   60, 1 };
      glLightfv(GL_LIGHT0 + world->nrOfRobots + i, GL_DIFFUSE,
		diffuseLight);
      glLightfv(GL_LIGHT0 + world->nrOfRobots + i, GL_POSITION,
		positionLight);
      glLightf(GL_LIGHT0 + world->nrOfRobots + i, GL_LINEAR_ATTENUATION,
	       0.001);
      glLightf(GL_LIGHT0 + world->nrOfRobots + i, GL_QUADRATIC_ATTENUATION,
	       0.0001);
      glEnable(GL_LIGHT0 + world->nrOfRobots + i);
   }
}

bool loadTGA(TextureImage * texture, char *filename, bool useMipMap)
{
   GLubyte TGAheader[12] = { 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 };	// Uncompressed TGA Header
   GLubyte TGAcompare[12];	// Used To Compare TGA Header
   GLubyte header[6];		// First 6 Useful Bytes From The Header
   GLuint bytesPerPixel;	// Holds Number Of Bytes Per Pixel Used In The TGA File
   GLuint imageSize;		// Used To Store The Image Size When Setting Aside Ram
   GLuint temp;			// Temporary Variable
   GLuint type = GL_RGBA;	// Set The Default GL Mode To RBGA (32 BPP)

   FILE *file = fopen(filename, "rb");	// Open The TGA File

   if (file == NULL ||		// Does File Even Exist?
       fread(TGAcompare, 1, sizeof(TGAcompare), file) != sizeof(TGAcompare) ||	// Are There 12 Bytes To Read?
       memcmp(TGAheader, TGAcompare, sizeof(TGAheader)) != 0 ||	// Does The Header Match What We Want?
       fread(header, 1, sizeof(header), file) != sizeof(header))	// If So Read Next 6 Header Bytes
   {
      if (file == NULL)		// Did The File Even Exist? *Added Jim Strong*
	 return false;		// Return False
      else {
	 fclose(file);		// If Anything Failed, Close The File
	 return false;		// Return False
      }
   }

   texture->width = header[1] * 256 + header[0];	// Determine The TGA Width      (highbyte*256+lowbyte)
   texture->height = header[3] * 256 + header[2];	// Determine The TGA Height     (highbyte*256+lowbyte)

   if (texture->width <= 0 ||	// Is The Width Less Than Or Equal To Zero
       texture->height <= 0 ||	// Is The Height Less Than Or Equal To Zero
       (header[4] != 24 && header[4] != 32))	// Is The TGA 24 or 32 Bit?
   {
      fclose(file);		// If Anything Failed, Close The File
      return false;		// Return False
   }

   texture->bpp = header[4];	// Grab The TGA's Bits Per Pixel (24 or 32)
   bytesPerPixel = texture->bpp / 8;	// Divide By 8 To Get The Bytes Per Pixel
   imageSize = texture->width * texture->height * bytesPerPixel;	// Calculate The Memory Required For The TGA Data

   texture->imageData = (GLubyte *) malloc(imageSize);	// Reserve Memory To Hold The TGA Data

   if (texture->imageData == NULL ||	// Does The Storage Memory Exist?
       fread(texture->imageData, 1, imageSize, file) != imageSize)	// Does The Image Size Match The Memory Reserved?
   {
      if (texture->imageData != NULL)	// Was Image Data Loaded
	 free(texture->imageData);	// If So, Release The Image Data

      fclose(file);		// Close The File
      return false;		// Return False
   }

   for (GLuint i = 0; i < int (imageSize); i += bytesPerPixel)	// Loop Through The Image Data
   {				// Swaps The 1st And 3rd Bytes ('R'ed and 'B'lue)
      temp = texture->imageData[i];	// Temporarily Store The Value At Image Data 'i'
      texture->imageData[i] = texture->imageData[i + 2];	// Set The 1st Byte To The Value Of The 3rd Byte
      texture->imageData[i + 2] = temp;	// Set The 3rd Byte To The Value In 'temp' (1st Byte Value)
   }

   fclose(file);		// Close The File

   if (texture->bpp == 24)	// Was The TGA 24 Bits
   {
      type = GL_RGB;		// If So Set The 'type' To GL_RGB
   }
   // Build A Texture From The Data
   glGenTextures(1, &texture->texID);	// Generate OpenGL texture IDs

   glBindTexture(GL_TEXTURE_2D, texture->texID);	// Bind Our Texture
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear Filtered
   if (useMipMap) {
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		      GL_LINEAR_MIPMAP_NEAREST);
      gluBuild2DMipmaps(GL_TEXTURE_2D, 3, texture->width, texture->height,
			GL_RGB, GL_UNSIGNED_BYTE, texture->imageData);
   } else {
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// Linear Filtered
      glTexImage2D(GL_TEXTURE_2D, 0, type, texture->width, texture->height,
		   0, type, GL_UNSIGNED_BYTE, texture->imageData);
   }

   return true;			// Texture Building Went Ok, Return True
}

GLvoid buildFont(TextureImage * texture)	// Build Our Font Display List
{
   fontBase = glGenLists(256);	// Creating 256 Display Lists
   glBindTexture(GL_TEXTURE_2D, texture->texID);	// Select Our Font Texture
   for (int loop1 = 0; loop1 < 256; loop1++)	// Loop Through All 256 Lists
   {
      float cx = float (loop1 % 16) / 16.0f;	// X Position Of Current Character
      float cy = float (loop1 / 16) / 16.0f;	// Y Position Of Current Character

      glNewList(fontBase + loop1, GL_COMPILE);	// Start Building A List
      glBegin(GL_QUADS);	// Use A Quad For Each Character
      glTexCoord2f(cx, 1.0f - cy - 1.0 / 16.0);	// Texture Coord (Bottom Left)
      glVertex2d(0, 16);	// Vertex Coord (Bottom Left)
      glTexCoord2f(cx + 1.0 / 16.0, 1.0f - cy - 1.0 / 16.0);	// Texture Coord (Bottom Right)
      glVertex2i(16, 16);	// Vertex Coord (Bottom Right)
      glTexCoord2f(cx + 1.0 / 16.0, 1.0f - cy - 0.001f);	// Texture Coord (Top Right)
      glVertex2i(16, 0);	// Vertex Coord (Top Right)
      glTexCoord2f(cx, 1.0f - cy - 0.001f);	// Texture Coord (Top Left)
      glVertex2i(0, 0);		// Vertex Coord (Top Left)
      glEnd();			// Done Building Our Quad (Character)
      glTranslated(11, 0, 0);	// Move To The Right Of The Character
      glEndList();		// Done Building The Display List
   }				// Loop Until All 256 Are Built
}

void showText(const char *fmt, ...)
{
   static char text[1024];	// Holds Our String
   va_list ap;			// Pointer To List Of Arguments

   if (fmt == NULL)		// If There's No Text
      return;			// Do Nothing

   va_start(ap, fmt);		// Parses The String For Variables
   vsprintf(text, fmt, ap);	// And Converts Symbols To Actual Numbers
   va_end(ap);			// Results Are Stored In Text

   textX = 6.0;
   textStr = text;
}

GLvoid opengl_print(double x, double y, double alpha, int set, char *text)	// Where The Printing Happens
{
   if (set > 1)			// Did User Choose An Invalid Character Set?
   {
      set = 1;			// If So, Select Set 1 (Italic)
   }

   glPushMatrix();
   glEnable(GL_BLEND);
   glListBase(fontBase - 32 + (128 * set));	// Choose The Font Set (0 or 1)

   glLoadIdentity();		// Reset The Modelview Matrix
   glNormal3f(0, 0, 1);
   glTranslatef(x, y, -20);
   glScalef(0.1f, 0.1f, 0.1f);
   glRotatef(180, 1, 0, 0);
   glBindTexture(GL_TEXTURE_2D, textures[4].texID);	// Bind Our Texture
   glColor4f(0.5, 0.5, 0, alpha);
   glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);	// Write The Text To The Screen

   glLoadIdentity();		// Reset The Modelview Matrix
   glTranslatef(x, y, -19.5);
   glScalef(0.1f, 0.1f, 0.1f);	// Make The Text 2X Taller
   glRotatef(180, 1, 0, 0);
   glColor4f(1, 1, 0, alpha);
   glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);	// Write The Text To The Screen
   glDisable(GL_BLEND);
   glPopMatrix();
}

gint opengl_key_press_callback(GtkWidget * widget,
			       GdkEventKey * event, gpointer data)
{
   char text[100];
   //      printf("key press %d!\n",event->keyval);
   switch (event->keyval) {
   case GDK_0:
   case GDK_section:
      camera.focusOnRobot = 0;
      showText("Robot 0");
      break;
   case GDK_1:
      if (world->nrOfRobots > 1) {
	 camera.focusOnRobot = 1;
	 showText("Robot 1");
      }
      break;
   case GDK_2:
      if (world->nrOfRobots > 2) {
	 camera.focusOnRobot = 2;
	 showText("Robot 2");
      }
      break;
   case GDK_3:
      if (world->nrOfRobots > 3) {
	 camera.focusOnRobot = 3;
	 showText("Robot 3");
      }
      break;
   case GDK_4:
      if (world->nrOfRobots > 4) {
	 camera.focusOnRobot = 4;
	 showText("Robot 4");
      }
      break;
   case GDK_5:
      if (world->nrOfRobots > 5) {
	 camera.focusOnRobot = 5;
	 showText("Robot 5");
      }
      break;
   case GDK_6:
      if (world->nrOfRobots > 6) {
	 camera.focusOnRobot = 6;
	 showText("Robot 6");
      }
      break;
   case GDK_7:
      if (world->nrOfRobots > 7) {
	 camera.focusOnRobot = 7;
	 showText("Robot 7");
      }
      break;
   case GDK_BackSpace:
      camera.focusOnRobot = -1;
      showText("Centered camera");
      break;
   case GDK_C:
      if (--camera.mode < 0)
	 camera.mode += 5;
      showText("Cameramode %d", camera.mode);
      break;
   case GDK_c:
      camera.mode = ++camera.mode % 5;
      if (camera.mode == 4)
	 camera.thirdDist = bigaxis;
      showText("Cameramode %d", camera.mode);
      break;
   case GDK_f:
   case GDK_F:
      floorLights = !floorLights;
      if (floorLights)
	 showText("Floorlights on");
      else
	 showText("Floorlights off");
      break;
   case GDK_i:
   case GDK_I:
      showIdentities = !showIdentities;
      if (showIdentities)
	 showText("Identities on");
      else
	 showText("Identities off");
      break;
   case GDK_l:
   case GDK_L:
      lighting = !lighting;
      if (lighting) {
	 glEnable(GL_LIGHTING);
	 showText("Lighting on");
      } else {
	 glDisable(GL_LIGHTING);
	 showText("Lighting off");
      }
      break;
   case GDK_b:
   case GDK_B:
      showRubberBands = !showRubberBands;
      if (showRubberBands)
	 showText("Rubberbands on");
      else
	 showText("Rubberbands off");
      break;
   case GDK_Left:
      camera.thirdAngle -= 10;
      if (camera.thirdAngle < 0)
	 camera.thirdAngle += 360;
      break;
   case GDK_Right:
      camera.thirdAngle += 10;
      if (camera.thirdAngle >= 360)
	 camera.thirdAngle -= 360;
      break;
   case GDK_Up:
      camera.thirdDist -= 20;
      if (camera.thirdDist < 20)
	 camera.thirdDist = 20;
      break;
   case GDK_Down:
      camera.thirdDist += 20;
      break;
   case GDK_Page_Up:
      camera.thirdHeight += 20;
      break;
   case GDK_Page_Down:
      camera.thirdHeight -= 20;
      if (camera.thirdHeight < 20)
	 camera.thirdHeight = 20;
      break;
   case GDK_z:
   case GDK_Z:
      showZones = !showZones;
      if (showZones)
	 showText("Zones shown");
      else
	 showText("Zones hidden");
      break;
   case GDK_r:
   case GDK_R:
      reflections = !reflections;
      if (reflections)
	 showText("Reflections on");
      else
	 showText("Reflections off");
      break;
   case GDK_plus:
      floorAlpha += 0.05;
      if (floorAlpha > 1.0)
	 floorAlpha = 1.0;
      sprintf(text, " %0.2f flooralpha", floorAlpha);
      showText(text);
      break;
   case GDK_minus:
      floorAlpha -= 0.05;
      if (floorAlpha < 0.0)
	 floorAlpha = 0.0;
      sprintf(text, " %0.2f flooralpha", floorAlpha);
      showText(text);
      break;
   case GDK_T:
   case GDK_t:
      trails = !trails;
      if (trails)
	 showText("Trails on");
      else
	 showText("Trails off");
      break;
   default:
      break;
   }
   return TRUE;
}

gint initopengl(GtkWidget * widget)
{
   printf(">>> initopengl\n");
   xt = (float)world->dx / -2.0f;
   yt = (float)world->dy / -2.0f;
   bigaxis = (float)world->dx;
   if (world->dy > world->dx)
     bigaxis = (float)world->dy;
   initCamera(&camera); // must be after init of bigaxis (uses it)
   for (int i = 0; i < ROBOT_TRAIL_LENGTH; i++) {
      for (int j = 0; j < 8; j++) {
	 trail[j][i].x = 0.0;
	 trail[j][i].y = 0.0;
	 trail[j][i].angle = 0.0;
      }
   }
   /* OpenGL functions can be called only if make_current returns true */
   if (gtk_gl_area_make_current(GTK_GL_AREA(widget))) {
      printf(">>> made gl area current\n");
      glViewport(0, 0, widget->allocation.width,
		 widget->allocation.height);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      glFrustum(-1, 1, -1, 1, 2, 5000);
      glClearColor(0.0f, 0.1f, 0.2f, 1.0f);
      glEnable(GL_CULL_FACE);
      glCullFace(GL_BACK);
      glShadeModel(GL_SMOOTH);

      glEnable(GL_COLOR_MATERIAL);
      glEnable(GL_TEXTURE_2D);

      glEnable(GL_DEPTH_TEST);	// Enables Depth Testing
      glDepthFunc(GL_LEQUAL);	// The Type Of Depth Testing To Do
      glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations

      //glBlendFunc(GL_SRC_ALPHA,GL_ONE);             // Blending Function For Translucency Based On Source Alpha Value ( NEW )
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	// Blending Function For Translucency Based On Source Alpha Value ( NEW )

      // don't mipmap the font or the floorlight (won't work)
      loadTGA(&textures[0], "textures/khepera.tga", true);	// robot top texture
      loadTGA(&textures[1], "textures/wall.tga", false);	// wall texture
      loadTGA(&textures[2], "textures/floor.tga", true);	// floor texture
      loadTGA(&textures[3], "textures/kheperaround.tga", true);	// side of khepera
      loadTGA(&textures[4], "textures/font.tga", false);	// font to write with
      loadTGA(&textures[5], "textures/star.tga", false);	// floorpiccy for lights
      loadTGA(&textures[6], "textures/energy.tga", true);	// small obstacle

      buildFont(&textures[4]);

      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
   }
   return TRUE;
}

void updateCamera()
{
   float rx = 0, ry = 0, rz = 25, ra = 0;
   if (camera.focusOnRobot >= 0) {
      rx = world->robot[camera.focusOnRobot]->getX() + xt;
      ry = (world->dy - world->robot[camera.focusOnRobot]->getY()) + yt;
      ra = world->robot[camera.focusOnRobot]->direction;
   }
   float newCamX, newCamY, newCamZ;
   float newFocusX, newFocusY, newFocusZ;
   float newUpX = 0, newUpY = 0, newUpZ = 1;	// up-vector of camera

   switch (camera.mode) {
   case 0:
      // fixed pos. straight down camera with focus on center
      newCamX = newCamY = newFocusX = newFocusY = newFocusZ = 0;
      newCamZ = bigaxis+10;
      newUpX = newUpZ = 0;
      newUpY = 1;
      camera.sloppy = true;
      break;
   case 1:
      // at south end focusing on robot
      newCamX = 0;
      newCamY = yt;
      newCamZ = -yt;
      newFocusX = rx;
      newFocusY = ry;
      newFocusZ = rz;
      camera.sloppy = true;
      break;
   case 2:
      // first person view
      newCamX = rx;
      newCamY = ry;
      newCamZ = rz + 15;	// on top of robot
      newFocusX = rx + 20.0 * cos(g_DGRtoRAD(ra));
      newFocusY = ry - 20.0 * sin(g_DGRtoRAD(ra));
      newFocusZ = rz + 5;	// look slightly down floor
      camera.sloppy = false;
      break;
   case 3:
      // third person view
      newCamX =
	  rx - camera.thirdDist * cos(g_DGRtoRAD(ra + camera.thirdAngle));
      newCamY =
	  ry + camera.thirdDist * sin(g_DGRtoRAD(ra + camera.thirdAngle));
      newCamZ = rz + camera.thirdHeight;
      newFocusX = rx;
      newFocusY = ry;
      newFocusZ = rz - 10;
      camera.sloppy = true;
      break;
   case 4:
      // rotating camera
      camera.thirdAngle -= 360.0 / 1000.0;
      newCamX = camera.thirdDist * cos(g_DGRtoRAD(camera.thirdAngle));
      newCamY = camera.thirdDist * sin(g_DGRtoRAD(camera.thirdAngle));
      newCamZ = camera.thirdDist;
      newFocusX = 0;
      newFocusY = 0;
      newFocusZ = 0;
      camera.sloppy = true;
      break;
   default:
      break;
   }

   if (camera.sloppy) {
      camera.camFactor -= 0.95 / 30.0;
      if (camera.camFactor < 0.05)
	 camera.camFactor = 0.05;
      camera.focusFactor -= 0.8 / 30.0;
      if (camera.focusFactor < 0.2)
	 camera.focusFactor = 0.2;
   } else {
      camera.camFactor += 0.95 / 30.0;
      if (camera.camFactor > 1.0)
	 camera.camFactor = 1.0;
      camera.focusFactor += 0.8 / 30.0;
      if (camera.focusFactor > 1.0)
	 camera.focusFactor = 1.0;
   }

   camera.x += camera.camFactor * (newCamX - camera.x);
   camera.y += camera.camFactor * (newCamY - camera.y);
   camera.z += camera.camFactor * (newCamZ - camera.z);
   camera.focusX += camera.focusFactor * (newFocusX - camera.focusX);
   camera.focusY += camera.focusFactor * (newFocusY - camera.focusY);
   camera.focusZ += camera.focusFactor * (newFocusZ - camera.focusZ);
   camera.upX += camera.camFactor * (newUpX - camera.upX);
   camera.upY += camera.camFactor * (newUpY - camera.upY);
   camera.upZ += camera.camFactor * (newUpZ - camera.upZ);

   gluLookAt(camera.x, camera.y, camera.z,
	     camera.focusX, camera.focusY, camera.focusZ,
	     camera.upX, camera.upY, camera.upZ);
}

void drawWall3d(Wall * wall)
{
   int segment, length;
   glPushMatrix();
   glBindTexture(GL_TEXTURE_2D, textures[1].texID);
   glTranslatef(xt + wall->x1, yt + (world->dy - wall->y1), WALL_HEIGHT);
   glRotatef(g_RADtoDGR(-atan2(wall->dy, wall->dx)), 0, 0, 1);
   length = (int)sqrt(wall->ddxy);
//      glColor3f(1,1,1);
   glBegin(GL_QUADS);
   /* TOP of wall */
   glNormal3f(0, 0, 1);
   for (segment = 0; (segment + WALL_SEGMENT) <= length;
	segment += WALL_SEGMENT) {
      glTexCoord2f(1, 1);
      glVertex2f(segment + WALL_SEGMENT, WALL_WIDTH);
      glTexCoord2f(0, 1);
      glVertex2f(segment, WALL_WIDTH);
      glTexCoord2f(0, LOW_TEX_WIDTH);
      glVertex2f(segment, -WALL_WIDTH);
      glTexCoord2f(1, LOW_TEX_WIDTH);
      glVertex2f(segment + WALL_SEGMENT, -WALL_WIDTH);
   }
   if ((length % WALL_SEGMENT) != 0) {
      glTexCoord2f(double (length % WALL_SEGMENT) / WALL_SEGMENT, 1);
      glVertex2f(length, WALL_WIDTH);
      glTexCoord2f(0, 1);
      glVertex2f(length - (length % WALL_SEGMENT), WALL_WIDTH);
      glTexCoord2f(0, LOW_TEX_WIDTH);
      glVertex2f(length - (length % WALL_SEGMENT), -WALL_WIDTH);
      glTexCoord2f(double (length % WALL_SEGMENT) / WALL_SEGMENT,
		   LOW_TEX_WIDTH);
      glVertex2f(length, -WALL_WIDTH);
   }
   //      if (!lighting)
#ifdef DARKER_WALL_SIDES
   glColor3f(0.7, 0.7, 0.7);
#endif
   /* SOUTH side */
   glNormal3f(0, -1, 0);
   for (segment = 0; (segment + WALL_SEGMENT) <= length;
	segment += WALL_SEGMENT) {
      glTexCoord2f(1, 1);
      glVertex3f(segment + WALL_SEGMENT, -WALL_WIDTH, 0);
      glTexCoord2f(0, 1);
      glVertex3f(segment, -WALL_WIDTH, 0);
      glTexCoord2f(0, LOW_TEX_HEIGHT);
      glVertex3f(segment, -WALL_WIDTH, -WALL_HEIGHT);
      glTexCoord2f(1, LOW_TEX_HEIGHT);
      glVertex3f(segment + WALL_SEGMENT, -WALL_WIDTH, -WALL_HEIGHT);
   }
   if (length % WALL_SEGMENT != 0) {
      glTexCoord2f(double (length % WALL_SEGMENT) / WALL_SEGMENT, 1);
      glVertex3f(length, -WALL_WIDTH, 0);
      glTexCoord2f(0, 1);
      glVertex3f(length - (length % WALL_SEGMENT), -WALL_WIDTH, 0);
      glTexCoord2f(0, LOW_TEX_HEIGHT);
      glVertex3f(length - (length % WALL_SEGMENT), -WALL_WIDTH,
		 -WALL_HEIGHT);
      glTexCoord2f(double (length % WALL_SEGMENT) / WALL_SEGMENT,
		   LOW_TEX_HEIGHT);
      glVertex3f(length, -WALL_WIDTH, -WALL_HEIGHT);
   }
   /* NORTH side */
   glNormal3f(0, 1, 0);
   for (segment = 0; (segment + WALL_SEGMENT) <= length;
	segment += WALL_SEGMENT) {
      glTexCoord2f(1, 1);
      glVertex3f(segment, WALL_WIDTH, 0);
      glTexCoord2f(0, 1);
      glVertex3f(segment + WALL_SEGMENT, WALL_WIDTH, 0);
      glTexCoord2f(0, LOW_TEX_HEIGHT);
      glVertex3f(segment + WALL_SEGMENT, WALL_WIDTH, -WALL_HEIGHT);
      glTexCoord2f(1, LOW_TEX_HEIGHT);
      glVertex3f(segment, WALL_WIDTH, -WALL_HEIGHT);
   }
   if (length % WALL_SEGMENT != 0) {
      glTexCoord2f(double (length % WALL_SEGMENT) / WALL_SEGMENT, 1);
      glVertex3f(length - (length % WALL_SEGMENT), WALL_WIDTH, 0);
      glTexCoord2f(0, 1);
      glVertex3f(length, WALL_WIDTH, 0);
      glTexCoord2f(0, LOW_TEX_HEIGHT);
      glVertex3f(length, WALL_WIDTH, -WALL_HEIGHT);
      glTexCoord2f(double (length % WALL_SEGMENT) / WALL_SEGMENT,
		   LOW_TEX_HEIGHT);
      glVertex3f(length - (length % WALL_SEGMENT), WALL_WIDTH,
		 -WALL_HEIGHT);
   }

   glNormal3f(-1, 0, 0);
   glVertex3f(0, -WALL_WIDTH, 0);
   glVertex3f(0, WALL_WIDTH, 0);
   glVertex3f(0, WALL_WIDTH, -WALL_HEIGHT);
   glVertex3f(0, -WALL_WIDTH, -WALL_HEIGHT);

   glNormal3f(1, 0, 0);
   glVertex3f(length, WALL_WIDTH, 0);
   glVertex3f(length, -WALL_WIDTH, 0);
   glVertex3f(length, -WALL_WIDTH, -WALL_HEIGHT);
   glVertex3f(length, WALL_WIDTH, -WALL_HEIGHT);
   glEnd();

#ifdef DARKER_WALL_SIDES
   glColor3f(1, 1, 1);
#endif

   glPopMatrix();
}

void drawRobot3d(Robot * rob, int robotNr)
{
   static GLuint robotDisplayList = 0;
   int i;
   GLfloat robotColors[] = { 1, 1, 1,	// white
      0.2, 0.2, 1,		// blue
      1, 0.1, 0.1,		// red
      0.1, 1, 0.1,		// green
      1, 1, 0.1,		// yellow
      1, 0.1, 1,		// purple
      0.1, 1, 1,		// opal
      1, 0.5, 0.2		// brownish
   };

   if (!robotDisplayList) {
      robotDisplayList = glGenLists(1);
      glNewList(robotDisplayList, GL_COMPILE);
      glBindTexture(GL_TEXTURE_2D, textures[0].texID);
      glNormal3f(0, 0, 1);
      int segments = 32;
      double angle = 0;
      glBegin(GL_POLYGON);
      for (i = 0; i < segments; i++) {
	 glTexCoord2f(0.5 + 0.5 * cos(angle), 0.5 + 0.5 * sin(angle));
	 glVertex3f(rob->getRadius() * cos(angle),
		    rob->getRadius() * sin(angle), ROBOT_HEIGHT);
	 angle += M_PI * 2.0 / segments;
      }
      glEnd();
      double stepSize = M_PI * 2.0 / segments;
      glBindTexture(GL_TEXTURE_2D, textures[3].texID);
      glBegin(GL_QUADS);
      double rad = rob->getRadius() * 0.95;
      for (i = 0; i < segments; i++) {
	 glNormal3f(cos(angle + stepSize), sin(angle + stepSize), 0.0);
	 glTexCoord2f(double (i + 1) / segments, 1);
	 glVertex3d(rad * cos(angle + stepSize),
		    rad * sin(angle + stepSize), ROBOT_HEIGHT);
	 glNormal3f(cos(angle), sin(angle), 0.0);
	 glTexCoord2f(double (i) / segments, 1);
	 glVertex3d(rad * cos(angle), rad * sin(angle), ROBOT_HEIGHT);
	 glNormal3f(cos(angle), sin(angle), 0.0);
	 glTexCoord2f(double (i) / segments, LOW_ROBOT_TEX_HEIGHT);
	 glVertex3d(rad * cos(angle), rad * sin(angle), 0);
	 glNormal3f(cos(angle + stepSize), sin(angle + stepSize), 0.0);
	 glTexCoord2f(double (i + 1) / segments, LOW_ROBOT_TEX_HEIGHT);
	 glVertex3d(rad * cos(angle + stepSize),
		    rad * sin(angle + stepSize), 0);
	 angle += stepSize;
      }
      glEnd();
      double wheelRadius = 9.0;
      int wheelSegments = 16;
      angle = 0.0;
      glNormal3f(0, -1, 0);
      glBegin(GL_POLYGON);
      for (i = 0; i < wheelSegments; i++) {
	 glTexCoord2f((12.0 / 256.0) + (12.0 / 256.0) * cos(angle),
		      (1.0 - (59.0 / 256.0)) +
		      (12.0 / 256.0) * sin(angle));
	 glVertex3f(wheelRadius * cos(angle), -rob->getRadius(),
		    wheelRadius + wheelRadius * sin(angle));
	 angle += M_PI * 2.0 / wheelSegments;
      }
      glEnd();
      angle = M_PI * 2.0;
      glNormal3f(0, 1, 0);
      glBegin(GL_POLYGON);
      for (i = 0; i < wheelSegments; i++) {
	 glTexCoord2f((12.0 / 256.0) + (12.0 / 256.0) * cos(angle),
		      (1.0 - (59.0 / 256.0)) +
		      (12.0 / 256.0) * sin(angle));
	 glVertex3f(wheelRadius * cos(angle), rob->getRadius(),
		    wheelRadius + wheelRadius * sin(angle));
	 angle -= M_PI * 2.0 / wheelSegments;
      }
      glEnd();
      glEndList();
   }

   for (i = 0; i < (ROBOT_TRAIL_LENGTH - 1); i++) {
      trail[robotNr][i].x = trail[robotNr][i + 1].x;
      trail[robotNr][i].y = trail[robotNr][i + 1].y;
      trail[robotNr][i].angle = trail[robotNr][i + 1].angle;
   }
   trail[robotNr][ROBOT_TRAIL_LENGTH - 1].x = rob->getX();
   trail[robotNr][ROBOT_TRAIL_LENGTH - 1].y = rob->getY();
   trail[robotNr][ROBOT_TRAIL_LENGTH - 1].angle = rob->direction;

   if (trails) {
      glEnable(GL_BLEND);
      for (i = 0; i < ROBOT_TRAIL_LENGTH; i += ROBOT_TRAIL_DRAW_STEP) {
	 glPushMatrix();
	 glTranslatef(xt + trail[robotNr][i].x,
		      yt + (world->dy - trail[robotNr][i].y), 1.0f);
	 glRotatef(-trail[robotNr][i].angle, 0, 0, 1);
	 glColor4f(robotColors[robotNr * 3],
		   robotColors[robotNr * 3 + 1],
		   robotColors[robotNr * 3 + 2],
		   double (i) / double (ROBOT_TRAIL_LENGTH));
	 glCallList(robotDisplayList);
	 glPopMatrix();
      }
      glDisable(GL_BLEND);
   }
   // the "real" robot body
   glPushMatrix();
   glTranslatef(xt + rob->getX(), yt + (world->dy - rob->getY()), 0);
   glRotatef(-rob->direction, 0, 0, 1);
   glColor3fv(&robotColors[robotNr * 3]);
   glCallList(robotDisplayList);
   glPopMatrix();

   if (showIdentities) {
      // robot label
      glPushMatrix();
      glTranslatef(xt + rob->getX(), yt + (world->dy - rob->getY()),
		   ZONE_HEIGHT - 1.0);
      glColor3f(1, 1, 0);
      glEnable(GL_BLEND);
      glNormal3f(0, 0, 1);
      glScalef(5, 5, 1);
      glBindTexture(GL_TEXTURE_2D, textures[4].texID);	// Bind Our Texture
      glRotatef(-rob->direction + 90, 0, 0, 1);
      glRotatef(180, 1, 0, 0);
      glTranslatef(-8, -8, 0);
      glCallList(fontBase - 32 + '0' + robotNr);
      glDisable(GL_BLEND);
      glPopMatrix();
   }

   GLfloat ambientLight[] = { 0.7f, 0.7f, 0.7f, 1.0f };
   GLfloat diffuseLight[4];	// ={5,5,4.5,1};
   //      GLfloat specularLight[]={0.3f,0.0f,0.0f,1.0f};
   GLfloat positionLight[] =
       { rob->getX() + xt, (world->dy - rob->getY()) + yt, 60.0f, 1.0f };
   glLightfv(GL_LIGHT0 + robotNr, GL_AMBIENT, ambientLight);
   for (i = 0; i < 3; i++)
      diffuseLight[i] = robotColors[robotNr * 3 + i] * 5.0;
   diffuseLight[3] = 1;
   glLightfv(GL_LIGHT0 + robotNr, GL_DIFFUSE, diffuseLight);
   //      glLightfv(GL_LIGHT0+robotNr,GL_SPECULAR,specularLight);
   glLightfv(GL_LIGHT0 + robotNr, GL_POSITION, positionLight);
   glLightf(GL_LIGHT0 + robotNr, GL_LINEAR_ATTENUATION, 0.001);
   glLightf(GL_LIGHT0 + robotNr, GL_QUADRATIC_ATTENUATION, 0.0001);
   glEnable(GL_LIGHT0 + robotNr);
}

void drawObstacle3d(Obst * obst)
{
   int i;
   glPushMatrix();
   glBindTexture(GL_TEXTURE_2D, textures[1].texID);
   glTranslatef(xt + obst->x, yt + (world->dy - obst->y), 0);
//      glColor3f(1,1,1);
   glNormal3f(0.0, 0.0, 1.0);
   int segments = 14;
   double angle = 0;
   double stepSize = M_PI * 2.0 / segments;
   glBegin(GL_POLYGON);
   for (i = 0; i < segments; i++) {
      glTexCoord2f(0.5 + 0.5 * cos(angle), 0.5 + 0.5 * sin(angle));
      glVertex3f(obst->r * cos(angle),
		 obst->r * sin(angle), WALL_HEIGHT + 1);
      angle += stepSize;
   }
   glEnd();
   angle = 0;
#ifdef DARKER_WALL_SIDES
   glColor3f(0.7, 0.7, 0.7);
#endif
   glBegin(GL_QUADS);
   for (i = 0; i < segments; i++) {
      glNormal3f(cos(angle + stepSize), sin(angle + stepSize), 0.0);
      glTexCoord2f(double (i + 1) / segments, 1);
      glVertex3d(obst->r * cos(angle + stepSize),
		 obst->r * sin(angle + stepSize), WALL_HEIGHT + 1);
      glNormal3f(cos(angle), sin(angle), 0.0);
      glTexCoord2f(double (i) / segments, 1);
      glVertex3d(obst->r * cos(angle), obst->r * sin(angle),
		 WALL_HEIGHT + 1);
      glNormal3f(cos(angle), sin(angle), 0.0);
      glTexCoord2f(double (i) / segments, LOW_TEX_HEIGHT);
      glVertex3d(obst->r * cos(angle), obst->r * sin(angle), 0);
      glNormal3f(cos(angle + stepSize), sin(angle + stepSize), 0.0);
      glTexCoord2f(double (i + 1) / segments, LOW_TEX_HEIGHT);
      glVertex3d(obst->r * cos(angle + stepSize),
		 obst->r * sin(angle + stepSize), 0);
      angle += stepSize;
   }
   glEnd();
#ifdef DARKER_WALL_SIDES
   glColor3f(1, 1, 1);
#endif
   glPopMatrix();
}

void drawZone3d(Zone * zone)
{
   int i;
   glPushMatrix();
   glEnable(GL_BLEND);
   glDisable(GL_TEXTURE_2D);
   glTranslatef(xt + zone->x, yt + (world->dy - zone->y), 0);
   glColor4f(0.3, 0.3, 0.9, 0.3);
   glNormal3f(0.0, 0.0, 1.0);
   int segments = 32;
   double angle = 0;
   double stepSize = M_PI * 2.0 / segments;
   glBegin(GL_POLYGON);
   for (i = 0; i < segments; i++) {
      glTexCoord2f(0.5 + 0.5 * cos(angle), 0.5 + 0.5 * sin(angle));
      glVertex3f(zone->r * cos(angle), zone->r * sin(angle), ZONE_HEIGHT);
      angle += stepSize;
   }
   glEnd();
   angle = 0;
   glColor4f(0.1, 0.1, 0.7, 0.15);
   glBegin(GL_QUADS);
   for (i = 0; i < segments; i++) {
      glNormal3f(cos(angle + stepSize), sin(angle + stepSize), 0.0);
      glVertex3d(zone->r * cos(angle + stepSize),
		 zone->r * sin(angle + stepSize), ZONE_HEIGHT);
      glNormal3f(cos(angle), sin(angle), 0.0);
      glVertex3d(zone->r * cos(angle), zone->r * sin(angle), ZONE_HEIGHT);
      glNormal3f(cos(angle), sin(angle), 0.0);
      glVertex3d(zone->r * cos(angle), zone->r * sin(angle), 0);
      glNormal3f(cos(angle + stepSize), sin(angle + stepSize), 0.0);
      glVertex3d(zone->r * cos(angle + stepSize),
		 zone->r * sin(angle + stepSize), 0);
      angle += stepSize;
   }
   glEnd();
   glEnable(GL_TEXTURE_2D);
   glDisable(GL_BLEND);
   glPopMatrix();
}

void drawSobst3d(Sobst * sObst)
{
   static GLuint sobstDisplayList = 0;
   int i;
   glPushMatrix();
   if (!sobstDisplayList) {
      sobstDisplayList = glGenLists(1);
      glNewList(sobstDisplayList, GL_COMPILE);
      glEnable(GL_BLEND);
      glBindTexture(GL_TEXTURE_2D, textures[6].texID);
      glNormal3f(0.0, 0.0, 1.0);
      int segments = 26;
      double angle = 0;
      double stepSize = M_PI * 2.0 / segments;
      glBegin(GL_POLYGON);
      for (i = 0; i < segments; i++) {
	 glTexCoord2f(57.0 / 256.0 + 57.0 / 256.0 * cos(angle),
		      57.0 / 256.0 + 57.0 / 256.0 * sin(angle));
	 glVertex3f(sObst->r * cos(angle),
		    sObst->r * sin(angle), SOBST_HEIGHT);
	 angle += stepSize;
      }
      glEnd();
      angle = 0;
      glBegin(GL_QUADS);
      for (i = 0; i < segments; i++) {
	 glNormal3f(cos(angle + stepSize), sin(angle + stepSize), 0.0);
	 glTexCoord2f(double (i + 1) / segments, 1);
	 glVertex3d(sObst->r * cos(angle + stepSize),
		    sObst->r * sin(angle + stepSize), SOBST_HEIGHT);
	 glNormal3f(cos(angle), sin(angle), 0.0);
	 glTexCoord2f(double (i) / segments, 1);
	 glVertex3d(sObst->r * cos(angle), sObst->r * sin(angle),
		    SOBST_HEIGHT);
	 glNormal3f(cos(angle), sin(angle), 0.0);
	 glTexCoord2f(double (i) / segments, LOW_SOBST_TEX_HEIGHT);
	 glVertex3d(sObst->r * cos(angle), sObst->r * sin(angle), 0);
	 glNormal3f(cos(angle + stepSize), sin(angle + stepSize), 0.0);
	 glTexCoord2f(double (i + 1) / segments, LOW_SOBST_TEX_HEIGHT);
	 glVertex3d(sObst->r * cos(angle + stepSize),
		    sObst->r * sin(angle + stepSize), 0);
	 angle += stepSize;
      }
      glEnd();
      glDisable(GL_BLEND);
      glEndList();
   }

   if (sObst->taken)
      glColor3f(1, 0.5, 0.5);
   else
      glColor3f(1, 1, 1);
   glTranslatef(xt + sObst->x, yt + (world->dy - sObst->y), 0);
   glCallList(sobstDisplayList);
   glPopMatrix();
}

void drawLight3d(Light * light)
{
   static int lightAngle = 0;
   glPushMatrix();
   glEnable(GL_BLEND);
   glBindTexture(GL_TEXTURE_2D, textures[5].texID);	// Select the star/light texture
   if (!lighting)
      glColor4f(1, 1, 1, 0.3);
   else
      glColor4f(1, 1, 1, 1);
   glNormal3f(0, 0, 1);
   glTranslatef(xt + light->x, yt + world->dy - light->y, 1);
   glBegin(GL_QUADS);
   glTexCoord2f(1, 1);
   glVertex2f(light->r, light->r);
   glTexCoord2f(0, 1);
   glVertex2f(-light->r, light->r);
   glTexCoord2f(0, 0);
   glVertex2f(-light->r, -light->r);
   glTexCoord2f(1, 0);
   glVertex2f(light->r, -light->r);
   glEnd();
   glRotatef(lightAngle--, 0, 0, 1);
   glTranslatef(0, 0, 1);
   glBegin(GL_QUADS);
   glTexCoord2f(1, 1);
   glVertex2f(light->r, light->r);
   glTexCoord2f(0, 1);
   glVertex2f(-light->r, light->r);
   glTexCoord2f(0, 0);
   glVertex2f(-light->r, -light->r);
   glTexCoord2f(1, 0);
   glVertex2f(light->r, -light->r);
   glEnd();
   glDisable(GL_BLEND);
   glPopMatrix();
}

void drawRubberBand3d(Robot * robot, Zone * zone)
{
   glPushMatrix();
   glTranslatef(xt + zone->x, yt + (world->dy - zone->y), 2);
   glRotatef(180 +
	     180 * -atan2(zone->y - robot->getY(),
			  zone->x - robot->getX()) / M_PI, 0, 0, 1);
   glColor3f(1, 1, 0);
   glNormal3f(0, 0, 1);
   glBegin(GL_QUADS);
   double length =
       sqrt(pow(zone->x - robot->getX(), 2) +
	    pow(zone->y - robot->getY(), 2));
   double width = 3;
   glVertex2f(length, width);
   glVertex2f(0, width);
   glVertex2f(0, -width);
   glVertex2f(length, -width);
   glEnd();
   glPopMatrix();
}

void drawFloor3d()
{
   glEnable(GL_BLEND);
   glPushMatrix();
   glBindTexture(GL_TEXTURE_2D, textures[2].texID);	// Select the floor texture
   glColor4f(1, 1, 1, floorAlpha);
   glNormal3f(0, 0, 1);
   float width = 5.0;	// 5 repetitions each way
   glBegin(GL_QUADS);
   glTexCoord2f(width, width);
   glVertex2f(xt + world->dx, yt + world->dy);
   glTexCoord2f(0, width);
   glVertex2f(xt, yt + world->dy);
   glTexCoord2f(0, 0);
   glVertex2f(xt, yt);
   glTexCoord2f(0 + width, 0);
   glVertex2f(xt + world->dx, yt);
   glEnd();
   glPopMatrix();
   glDisable(GL_BLEND);
}

gint opengl_configure(GtkWidget * widget, GdkEventConfigure * event)
{
   int w = widget->allocation.width;
   int h = widget->allocation.height;
   if (gtk_gl_area_make_current(GTK_GL_AREA(widget))) {
      glViewport(0, 0, w, h);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      double ratio = double (w) / h;
      glFrustum(-ratio, ratio, -1, 1, 2, 5000);
      glMatrixMode(GL_MODELVIEW);
   }
   return TRUE;
}

/* When widget is exposed its contents are redrawn. */
gint drawopengl(GtkWidget * widget, GdkEventExpose * event)
{
   int length;
   int i;
   static GLuint worldDisplayList = 0;

   //printf(">>> drawopengl\n");

   /* Draw only last expose. */
   if (event->count > 0)
      return TRUE;

   /* OpenGL functions can be called only if make_current returns true */
   if (gtk_gl_area_make_current(GTK_GL_AREA(widget))) {
      if (!initialized) {
	 initopengl(widget);
	 initialized = TRUE;
      }
      if (!worldDisplayList) {
	 worldDisplayList = glGenLists(1);
	 glNewList(worldDisplayList, GL_COMPILE);
	 for (int wallNr = 0; wallNr < world->getNrOfWalls(); wallNr++)
	    drawWall3d(world->wall[wallNr]);
	 for (int obstNr = 0; obstNr < world->getNrOfObst(); obstNr++)
	    drawObstacle3d(world->obst[obstNr]);
	 glEndList();
      }

      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
      glMatrixMode(GL_MODELVIEW);

      glLoadIdentity();

      updateCamera();

      if (floorLights) {
	 for (int lightNr = 0; lightNr < world->getNrOfLights(); lightNr++)
	    drawLight3d(world->light[lightNr]);
      }

      if (reflections) {
	 // reflection in floor
	 glPushMatrix();
	 glScalef(1, 1, -0.6);	// "reflect" across Z plane and compress
	 glCullFace(GL_FRONT);
	 glColor4f(1, 1, 1, 0.3);
	 glEnable(GL_BLEND);
	 glBlendFunc(GL_SRC_ALPHA, GL_ZERO);	// Blending Function For Translucency Based On Source Alpha Value ( NEW )
	 glCallList(worldDisplayList);	// floors, walls and static obstacles
	 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	// Blending Function For Translucency Based On Source Alpha Value ( NEW )
	 glDisable(GL_BLEND);
	 for (int sObstNr = 0; sObstNr < world->getNrOfSObst(); sObstNr++)
	    drawSobst3d(world->sobst[sObstNr]);
	 for (int robotNr = 0; robotNr < world->nrOfRobots; robotNr++)
	    drawRobot3d(world->robot[robotNr], robotNr);
	 if (showZones) {
	    for (int zoneNr = 0; zoneNr < world->getNrOfZones(); zoneNr++)
	       drawZone3d(world->zone[zoneNr]);
	 }
	 glCullFace(GL_BACK);
	 glPopMatrix();
      }

      drawFloor3d();		// put here to handle floor reflections

      glColor4f(1, 1, 1, 1);
      glCallList(worldDisplayList);	// floors, walls and static obstacles
      for (int sObstNr = 0; sObstNr < world->getNrOfSObst(); sObstNr++)
	 drawSobst3d(world->sobst[sObstNr]);
      if (showRubberBands) {
	if(world->getNrOfZones() >= world->nrOfRobots)
	  for (int robotNr = 0; robotNr < world->nrOfRobots; robotNr++)
	    drawRubberBand3d(world->robot[robotNr], world->zone[robotNr]);
      }
      for (int robotNr = 0; robotNr < world->nrOfRobots; robotNr++)
	 drawRobot3d(world->robot[robotNr], robotNr);
      if (showZones) {
	 for (int zoneNr = 0; zoneNr < world->getNrOfZones(); zoneNr++)
	    drawZone3d(world->zone[zoneNr]);
      }


      if (textStr) {
	 textX -= 0.3;
	 if (textX <= -1.0 * strlen(textStr)) {
	    textX = 6.0;
	    textStr = NULL;
	 } else {
	    double alpha = (textX + strlen(textStr)) / strlen(textStr);
	    alpha = CLAMP(alpha, 0.0, 1.0);
	    opengl_print(textX, 8, alpha, 0, textStr);
	 }
      }

      showLamps();

      /* Swap backbuffer to front */
      gtk_gl_area_swapbuffers(GTK_GL_AREA(widget));
   }

   return TRUE;
}
#endif	/* OPENGL */
