/*
  YAKS, a Khepera simulator including a separate GA and ANN 
  (Genetic Algoritm, Artificial Neural Net).
  Copyright (C) 2000  Johan Carlsson (johanc@ida.his.se)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#ifndef LIGHT_H
#define LIGHT_H
#include <stdio.h>
#include <stdlib.h>
#include "geom.h"
#include "param.h"

/**
 * @class Light light.h
 * @brief This class implements light sources
 *
 * Light sources are as all other elements implemented using lookup tables
 *
 */
class Light{
 public:
  Light(FILE *fp);
  Light(char *str);
  ~Light();
  void init(char *fileName);
  /** Light source centrum \a x coordinate */
  float x;
  /** Light source centrum \a y coordinate */
  float y;
  float *sensorVal(float kx, float ky, float kangle, float robRadius);
  /** Shared, light radius */
  static float r;
  /** Shared lookup table for sensor readings */
  static float rlight[27][180][8];        /* x light (180 degrees and 27 distances) */
 private:
  int relangle(float,float,float);
  /** Shared distance from which obstacles gives maximum sensor values */
  static float MinLightDistance;
  /** Shared step size, resolution for sensor values */
  static float step;
};

#endif /* LIGHT_H */
 
