/*
  YAKS, a Khepera simulator including a separate GA and ANN 
  (Genetic Algoritm, Artificial Neural Net).
  Copyright (C) 2000  Johan Carlsson (johanc@ida.his.se)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "monitor.h"

Monitor::Monitor(int iWW, int iWH, int pens){
  nrOfWatches = 0;
  display = NULL;
  watch = NULL;
  wH = iWH;
  wW = iWW;
  rulers = 0;
  timeStep=1;
  colorText = GUI_YELLOW;
  colorRuler = GUI_BLUE;
  colorBorder = GUI_BLACK;
  colorBG = GUI_WHITE;
  colorPen = 0;
  pen = new int(pens);
  nrOfPens = pens;
}

Monitor::~Monitor(){
  if(watch){
    for(int i = 0;i < nrOfWatches; i++)
      delete(watch[i]);
    free(watch);
  }
  delete(pen);
}

void Monitor::setPen(int i, int color){
  if(i>=0 && i < nrOfPens)
    if(color >= 0 && color < 256)
      pen[i]=color;
}
void Monitor::setBorder(int color){
   if(color >= 0 && color < 256)
     colorBorder=color;
}
void Monitor::setRuler(int color){
   if(color >= 0 && color < 256)
     colorRuler=color;
}
void Monitor::setText(int color){
   if(color >= 0 && color < 256)
     colorText=color;
}

void Monitor::setWatch(int index, double val){
  if(index >= 0 && index < nrOfWatches){
    watch[index]->setVal(val);
  }
}

void Monitor::addWatch(const char *title, const double min, const double max){
  if(display == NULL){
    watch = (Value**) realloc(watch,sizeof(Value*)*(1+nrOfWatches));
    watch[nrOfWatches++] = new Value(title,min,max);
  }
}

void Monitor::toggleRuler(){
  if(rulers)
    rulers = 0;
  else
    rulers = 1;
}

void Monitor::useNextPen(){
  colorPen++;
  if(colorPen == nrOfPens)
    colorPen = 0;
}

void Monitor::createWindow(){
  int offset;
  char title[12];
  static int monId=0;
  if(!display){
    g_snprintf(title,12,"Monitor%d",monId++);
    display = gui_window_new(wW,wH,title);
    gui_window_clear(display,1);
    gui_window_set_color(display,colorText);
    offset = (wH / nrOfWatches);
    for(int i=0; i < nrOfWatches; i++){
      if(watch[i]->getDescription())
 	gui_window_text(display,0,(int)(offset*i + offset / 2),watch[i]->getDescription());
    }
    gui_window_flush(display);
  }
}

void Monitor::drawValues(){
   int monHeight;
   double value;
   if(gfxUpdate){
     if(display != NULL){
       monHeight =(int)(wH / nrOfWatches);
       if(timeStep >= wW)
	 timeStep = 1;
       gui_window_set_color(display,GUI_WHITE);
       gui_window_line(display,timeStep,0,timeStep,wH);
       gui_window_set_color(display,pen[colorPen]);
       
       for(int i=1; i <= nrOfWatches; i++){
	 value = watch[i-1]->getNormal() * 0.9;
	 gui_window_line(display,timeStep,(gint)(monHeight*i-monHeight*value),
			 timeStep,(gint)(monHeight*i));
       }
       gui_window_set_color(display,colorRuler);
       gui_window_line(display,timeStep+1,0,timeStep+1,wH);
       if(rulers){ 
	 if(timeStep%50 == 0){
	   timeStep++;
	   gui_window_set_color(display,colorRuler);
	   gui_window_line(display,timeStep,0,timeStep,wH);
	 }
       }
       
       gui_window_flush(display);
       timeStep++;
     }
   }
}
