/*
  YAKS, a Khepera simulator including a separate GA and ANN 
  (Genetic Algoritm, Artificial Neural Net).
  Copyright (C) 2000  Johan Carlsson (johanc@ida.his.se)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "real.h"


#ifndef WIN32

int Real::serialLine[50];
#else

#include "Serial.h"
CSerial Real::serialLine[1];
#endif

Real::Real(){
  currBot = -1;
  for(int i=0; i < paramNrOfRobots; i++){
    for(int s=0; s < NETWORK_INPUTS; s++){
      input[i][s] = 0.0;
      simoutput[i][s] = 0.0;
    }
  }
}

Real::~Real(){
}

boolean Real::init(){
  int fail=0;
  if(useRadioTurrets){
    fail = initSerial(0,1);
    printf("Radio turret initialized\n");
  }else{
    for(int i=0; i < paramNrOfRobots; i++){
      if(initSerial(i,0))
	fail = 1;
    }
  }
  if(!fail){
    return(FALSE);
  }else{
    return(TRUE);
  }
}
#ifndef WIN32
boolean Real::initSerial(int robotNr,int radioBase){
  struct termios term;
  static long int speedkey;
  int fd;
  serialLine[robotNr] = open(serialDevice[robotNr], O_RDWR | O_EXCL);
  if(serialLine[robotNr] < 0){
    perror("Real::init() serial open");
    return FALSE;
  }
  fd = serialLine[robotNr];
  if (tcgetattr(fd,&term)!=0)
  {
    perror("Real::init() getting attributes");
    return FALSE;
  }
  switch(baudRate[robotNr])
  {
    case 9600:	speedkey=B9600; break;
    case 19200:	speedkey=B19200; break;
    case 38400:	speedkey=B38400; break;
    default:	
      fprintf(stderr,"Real::serialInit wrong baudrate falling back on 9600\n");
      speedkey=B9600; break;
  }
  cfsetispeed(&term,speedkey);  /* Speed */
  cfsetospeed(&term,speedkey);
  clear(term.c_iflag,            /* Input modes */
        IGNBRK|BRKINT|IGNPAR|INPCK|ISTRIP|ICRNL|INLCR|IXON|IXOFF);
  set(term.c_iflag,0);
  clear(term.c_oflag, OPOST);    /* Output modes */
  set(term.c_oflag,0);

#ifdef __STRICT_ANSI__
  clear(term.c_cflag,CSIZE|PARENB);     /* Control modes */
  set(term.c_cflag,CS8|CSTOPB);

  clear(term.c_lflag,ISIG|ICANON|ECHO|IEXTEN);
#else
  if(radioBase){
    clear(term.c_cflag,CSIZE|PARENB);     /* Control modes */
    set(term.c_cflag,CS8|CSTOPB|CRTSCTS);
  }else{
    clear(term.c_cflag,CSIZE|PARENB|CRTSCTS);     /* Control modes */
    set(term.c_cflag,CS8|CSTOPB);
  }
  /* Line modes */
#ifndef __FreeBSD__ /* FreeBSD does not like XCASE */
  clear(term.c_lflag,ISIG|ICANON|XCASE|ECHO|IEXTEN);
#else
  clear((term.c_lflag,ISIG|ICANON|ECHO|IEXTEN);
#endif
#endif
  set(term.c_lflag,0);   /* MIN and TIME */
  term.c_cc[VMIN]  = 0;
  term.c_cc[VTIME] = KHEP_DEFAULT_TIMEOUT;
  if (tcsetattr(fd,TCSANOW,&term)!=0)
  {
    perror("REAL::init() setting attributes");
    return FALSE;
  }
  return TRUE;
}
#else // win32
boolean Real::initSerial(int robotNr,int radioBase){
	DWORD rec;
	serialLine[robotNr].Open("COM1",0,0); // no input/output buffers
	serialLine[0].Setup(CSerial::EBaudrate(38400),CSerial::EDataBits(8),
		CSerial::EParity(NOPARITY),CSerial::EStopBits(TWOSTOPBITS));
	serialLine[0].SetupReadTimeouts (CSerial::EReadTimeoutNonblocking);

	serialLine[0].Write("\n\n",&rec,0,INFINITE);
	serialDrain(0,FALSE);
	return(TRUE);
}
#endif

#ifndef WIN32
int Real::readline(int fd, char *buffer, int bsize){
   int nRd,rSize;
   char c;
   rSize = 0;
   do{
     nRd=read(fd,&c,1);

     if(nRd < 0)
       break;
     buffer[rSize] = c;
     rSize+=nRd;
   }while(nRd==1 && rSize < bsize && c != '\n');
   buffer[rSize]='\0';
#ifdef REAL_DEBUG
   printf("%s\n",buffer);
#endif
   if(nRd>=0) 
     return rSize;
   else{
     perror("Real::readline()");
     return nRd;
   }
   
}
#else
int Real::readline(int dontCare, char *buffer, int bsize){
#ifdef REAL_DEBUG
	printf("### Enter readline \n");
#endif
   int rSize;
   DWORD nRd;
   char c;
   rSize = 0;
   do{
     //nRd=read(fd,&c,1);
	serialLine[0].Read(&c,1,&nRd,0,INFINITE);	// timeout in ms
     if(nRd < 0)
       break;
     buffer[rSize] = c;
     rSize+=nRd;
   }while(nRd==1 && rSize < bsize && c != '\n');
   buffer[rSize]='\0';
#ifdef REAL_DEBUG
   printf(" Read [%s]=>buffer\n",buffer);
#endif
   if(nRd>=0){
#ifdef REAL_DEBUG
	printf("### Leave ok readline \n");
#endif
     return rSize;
   }
   else{
#ifdef REAL_DEBUG
	printf("### Leave with error readline \n");
#endif

     return rSize;
   }
   
}
/*		DWORD nRd;
	serialLine[0].Read(buffer,bsize,&nRd,0,INFINITE);
//	printf("%s\n",buffer);
	return(nRd);
}	*/
#endif

void Real::serialDrain(int fd ,boolean verbose)
{
  char answer[100];
  int  rsize;
  do
  {
    rsize = readline(fd,answer,strlen(answer));
#ifdef REAL_DEBUG
    printf("Real::serialDrain draining %d bytes",rsize);
#endif
  }
  while (rsize > 0); /* Read till nothing more to read */
}

/* Khepera package routines                                                 */
/* These routine make a specific use of the serial management functions to  */
/* provide a more complex tool to communicate with a Khepera                */

/* Send line send           */
/* and receive line receive */
/* without any check control*/
boolean Real::khepSendRecv(int fd,char *send,char *receive,int recv_len)
{
  int rd;
#ifndef WIN32
  write(fd,send,strlen(send));
#else
	DWORD sent;
	serialLine[0].Write(send,&sent,0,INFINITE); // timeout in ms
#endif

  if ((rd = readline(fd,receive,recv_len)) > 0 ) return(TRUE);
  else
  {
    if (rd < 0) perror("Real::khepSendRecv()  read error");
    else perror("Real::khepSendRecv()  khepera not responding");
    return(FALSE);
  }
}


/* Send line send and receive line       */
/* receive with check control :          */
/* The first character of the received   */
/* line is the lower first character of  */
/* the send line.                        */
/* Try to have the good answer           */
/* KHEP_NB_RETRY times                   */
boolean Real::khepTalk(int fd,char *send,char *receive,int recv_len){
  int  rd,rsize,tries = 0;
#ifdef WIN32
  DWORD sent;
#endif
  char respond = tolower(send[0]);
#ifdef REAL_DEBUG
  printf("### khepTalk try to send :%s",send);
#endif
  do{
#ifndef WIN32
    write(fd,send,strlen(send));
#else
	serialLine[0].Write(send,&sent,0,INFINITE);
#endif
    rd = readline(fd,receive,recv_len);
    if (rd < 0){ /* File error */
      printf("### khepTalk did not get an answear\n");
      break;
    }
    else if (rd == 0) /* No response */
      tries++;
    else if (receive[0] != respond){ /* Incorrect answer */
      printf("### khepTalk did not except answear=[%s]\n",receive);
      serialDrain(fd,FALSE);
      tries++;
    }
    else{ /* Correct answer */
      if ((rsize = strlen(receive)) >= 2){ /* Strip terminating '\r\n' if any */
	if (receive[rsize-1] == '\n') receive[rsize-1] = '\0';
	if (receive[rsize-2] == '\r') receive[rsize-2] = '\0';
      }
      return(TRUE);
    }
  }while (tries < KHEP_NB_RETRY);
  if (rd == 0) 
    printf("Real::khepTalk() Khepera not responding\n");
  return(FALSE);
}

int Real::getSerialLine(int robotNr){
  char command[30];
  int fd;
#ifdef WIN32
  DWORD written;
#endif
  if(robotNr > paramNrOfRobots){
    fprintf(stderr,"Real::getSerialLine index [robotNr] out of range\n");
    exit(EXIT_FAILURE);
  }
  if(useRadioTurrets){
    if(robotNr!=currBot){
      sprintf(command,"*%d\n",robotNr+1);
#ifndef WIN32
      write(serialLine[0],command,strlen(command));
#else
	  serialLine[0].Write(command,&written,0,INFINITE);
#endif
    }
    currBot = robotNr;
#ifndef WIN32
    fd = serialLine[0];
#else
	fd = 0;
#endif
  }else{
#ifndef WIN32
    fd = serialLine[robotNr];
#else
	fd = 0;
#endif
  }
  return(fd);
}	
	   
void Real::setInput(int robotNr){
  int fd=0;
  fd = getSerialLine(robotNr);
  if(!setInputReal(robotNr,fd))
    fprintf(stderr,"Real::setInput() failed for robot %d\n",robotNr);
}

void Real::runStep(double mot1, double mot2, int robotNr){
  int fd;
  fd = getSerialLine(robotNr);
  if(!runStepReal(mot1,mot2,fd))
    fprintf(stderr,"Real::runStep() failed for robot %d\n",robotNr);

}

int Real::runStepReal(double mot1, double mot2, int fd){
  char buff[200];
  char command[15];
#ifndef WIN32
  sprintf(command,"D,%d,%d\n",int((double(mot1*20)-10)),
	  int((double(mot2*20)-10)));
#else
  sprintf(command,"D,%d,%d\n",int((double(mot1*20)-10)),
	  int((double(mot2*20)-10)));
#endif
  if(!khepTalk(fd,command,buff,199)){
    return(1);
  }
  return(0);
}

void Real::nextIndivid(){
  char buff[200];
#ifndef WIN32
  if(!khepTalk(serialLine[0],"D,0,0\n",buff,199))
    fprintf(stderr,"Real::nextIndivid() failed to stop robot\n");
#else
  if(!khepTalk(0,"D,0,0\n",buff,199))
   fprintf(stderr,"Real::nextIndivid() failed to stop robot\n");
#endif
}

int Real::setInputReal(int robotNr,int fd){
 static char buff[200];
 int kalle[8];
 buff[0] = '\0';
if(khepTalk(fd,"N\n",buff,199)){
   sscanf(buff,"n, %d, %d, %d, %d, %d, %d, %d, %d",
	  &kalle[0],
	  &kalle[1],
	  &kalle[2],
	  &kalle[3],
	  &kalle[4],
	  &kalle[5],
	  &kalle[6],
	  &kalle[7]);
   for(int i=0; i < 8; i++){
     input[robotNr][i]=((double)kalle[i]/1024.0);
     simoutput[robotNr][i] = ((double)kalle[i]/1024.0);
   }
   return(1);
 }
 return(0);
}
