/*
  YAKS, a Khepera simulator including a separate GA and ANN 
  (Genetic Algoritm, Artificial Neural Net).
  Copyright (C) 2000  Johan Carlsson (johanc@ida.his.se)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef TERMINAL_H
#define TERMINAL_H

#include <stdio.h>
#include <glib.h>
/** @struct CommandS terminal.h
 *  @brief Structure used for holding a user added command.
 */
struct CommandS{
  /** The command string. */
  char *command;
  /** Description of the command. */
  char *description;
  /** Pointer to next command. */
  CommandS *next;
  /** Pointer to the command function. */ 
  void (*func)(void);
};

/**
 * @class Terminal terminal.h
 * @brief Implements a user interactive terminal.
 */
class Terminal{
 public:
  Terminal();
  ~Terminal();
  void addItem(const char*,const char*, void(*)(void));
  void runTerminal();
  void setPrompt(const char *);
  void printMenu();
 protected:
 private:
  void runInternalCommand(const char*);
  void runUserCommand(const char*);
  void upCase(char*);
  /** Internal linked list of user commands. */
  CommandS *commands;
  /** The internal prompt. */
  char *prompt;
};



#endif /* TERMINAL_H */
