/*
  YAKS, Optann, a Khepera simulator including a separate GA and ANN 
  (Genetic Algoritm, Artificial Neural Net).
  Copyright (C) 2000  Johan Carlsson (johanc@ida.his.se)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#ifndef NEURON_H
#define NEURON_H
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <glib.h>
#ifdef BIT_WEIGHTS
typedef gint32 Weight;
// Max is 31 
#define WEIGHTBITS 8
#define W_TRANSLATE (1<<(WEIGHTBITS-1))
#else
typedef gdouble Weight;
#endif

typedef enum {INTERN,OUTPUT,INPUT,BIAS,SCN}NODE_T;
typedef enum {NORMAL,NULLAD,SCN_L}LINK_TO_T;
typedef enum {SIGMOID,STEP,LINEAR,NONE,LOGISTIC,TANH}ACTIVATION_T;

typedef struct{
  Weight weight;
  double *value;
  class Neuron *fromNode;
  class Neuron *toNode;
  class Neuron *SCN;
  gint32 fromId;
  gint32 toId;
  gint32 SCNid;
  LINK_TO_T typ;
  double slowness;
}LINK_T;

/** @class Neuron neuron.h
 * @brief The class implements a Neural Net neuron.
 *
 * @see Ann
 */
class Neuron{
 public:
  Neuron(int iLayerPriority,NODE_T inType,ACTIVATION_T iFuncT);
  Neuron(int iLayerPriority,NODE_T inType,ACTIVATION_T iFuncT,gint32 iId);
  ~Neuron();
  class Neuron*  duplicate();
  void mutateWeights(double percentOfBits);
  void mutateWeightsWithPri(double bitMutateProbability, int pri);
  int getPriority();
  double* activationPointer();
  void createLink(class Neuron *toNeuron, Weight iWeight,LINK_TO_T iType);
  void removeLink(int index);
  void removeLinksTo(class Neuron *toNeuron);
  void removeLinksTo(gint32  toNeuron);
  void correctAllLinksFrom(class Neuron *toNeuron);
  gint32 getId();
  NODE_T getNodeType();  
  ACTIVATION_T getActFunc();
  Weight getLinkWeight(int index);
  Weight *getLinkWeightPointer(int index);
  LINK_TO_T getLinkType(int index) const;
  gint32 getLinkToId(int index);
  gint32 getNextLinkId();
  gint32 getLinkSCNid(int index);
  int getNrOfLinks();
  void setLinkWeight(int index, Weight iweight);
  void activate();
  double getActivationValue();
  void setActivationValue(double iValue);
  int getWriteActivation();
  int SCNNotUsed() const;
  void createSCNLink(class Neuron *toNeuron,class Neuron *SCNNeuron);
 private:
  void setSCNUsed();
  double sigmoid(double in);
  double step(double in);
  double linear(double in);
  double logistic(double in);
  LINK_T **links;
  ACTIVATION_T actType;
  NODE_T nType;
  int nrOfLinks;
  int layerPriority;
  double *activationValue;
  double inValue;
  gint32 id;
  int linkCount;
  int scnUsed;

};

#endif /* NEURON_N */ 

  
