/*
  YAKS, a Khepera simulator including a separate GA and ANN 
  (Genetic Algoritm, Artificial Neural Net).
  Copyright (C) 2000  Johan Carlsson (johanc@ida.his.se)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#ifndef OBST_H
#define OBST_H
#include <stdio.h>
#include "param.h"
#include "geom.h"


/**
 * @class Obst obst.h
 * @brief This class implements round obstacles that cant be moved.
 *
 * Note that all round obstacles look the same. So they all share
 * the same radius and lookup table (declared static).
 */
class Obst{
 public:
  Obst(FILE *fp);
  Obst(char *str);
  ~Obst();
  void init(char *fileName);
  void save(char* fileName);
  void setRadius(float rad);
  float *sensorVal(float kx, float ky, float kangle, float robRadius);
  /** Obstacle \a x coordinate */ 
  float x;
  /** Obstacle \a y coordinate */ 
  float y;
  /** Shared, round obstacles radius */
  static float r; 
 private:
  int relangle(float,float,float);
  /** Shared obstacle range for sensors */
  static float MaxObstacleDistans;
  /** Shared distance from which obstacles gives maximum sensor values */
  static float MinObstacleDistans;
  /** Shared step size, resolution for sensor values */
  static float Step;
  /** Shared lookup table for sensor readings */
  static float obst[20][180][8];        
};


#endif /* OBST_H */
