/*
  Optann, a Khepera simulator including a separate GA and ANN 
  (Genetic Algoritm, Artificial Neural Net).
  Copyright (C) 2000  Johan Carlsson (johanc@ida.his.se)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#include "param.h"
#include <glib.h>
#include <string.h>
/* Serial thingis*/
/** Option, use of serial line (binary). */
int     useSerialLine=0; 
/** Option, use of radio base (radio turrets) */
int     useRadioTurrets=0;
/** List of serial devices to use, if \a useSerialDevice */
char    **serialDevice; 
/** List of baud rates, one for each for \a serialDevice */ 
int     *baudRate;      
/** List of strings describing the robots */
char    **robotStr;
/** Option, number of robots to simulate. */
int     paramNrOfRobots=1; 
/** Option, Delay for gui update @obselute */
int     updateDelay= 1000;
/** Option, scaling of simulator window. */
float   xScale=0.5;            
/** Global, trajactory robot. */
int     notTrajactoryRobot=0;  
/** Option, selection method to use (in main simulation loop). */
int     selectionMethod=0;     
/** Option, the probability that a bit will mutate. */
double     bitMutateProbability=2;
/** Global, random seed */
int	seed=0;                
/** Option, number of epochs to run each individ. */
int     amoebaEpochs=10;       
/** Option, number of generations to run simulation. */
int     nrOfGenerations=10;    
/** Option, number of individs for each generation. */
int     nrOfIndivids=100;
/** Option, number of kids for each father. */
int     nrOfKids=5;
/** Option, number of fathers that make kids for each generation. */
int     nrOfFathers=20;
/** Option, start generation (>=0 means load weights). */
int     startGeneration = -1;  
/** Option, verbose level (0 - 1 - 2, used binary for now). */
int     verboseLevel=1;        
/** Option, save only the best N amoebas each generation. */
int     saveNrOfIndivids;      
/** Option, noise on sensors (binary). */
int     sensorNoise=0;         
/** Option, amount of noise to add to sensor inputs. */
float   amountOfNoise=0.1;     
/** Option, test start from the same position (binary). */
int     testSameStartPosition=0;
/** Option, fitness function to use (in main simulation loop). */ 
int     ffitness=1;            
/** Option, use noise on rod sensor (binary). */
int     useRodNoise=0;         
/** Option, use noise on gripper sensor (binary). */
int     gripperNoise=1;        
/** Option, nr of time steps to run each epoch (100 ms/step). */
int     nrOfSteps = 200;       
/** Option, reposition obstacles between epochs. */
int     moveObstacles=1;       
/** Global, GUI simulator area update (binary). */
int     gfxUpdate=1;
/** Global, wait on start */
int		waitOnStart=1;
/** Global, GUI update of user windows (binary). */
int     userUpdate=1;          
/** USED ?!? */
float   energyDecrease=0;      
/** Option, nr of individs to log fitness for after each generation. */
int     nrOfIndividsToLog=1;   
/** Option, save weights of each x'th generation. */
int     saveGeneration=100;    
/** Option, filename of the picture to use for drawing of robots (not used). */
char    robotPicFile[100];     
/** Option, filename of the picture to use for drawing the world (not used). */
char    worldPicFile[100];     
/** Option, path to where logs should be saved. */
char    logPath[100];          
/** Option, path to the world file. */
char    worldPath[100];       
 /** Option, path to the camera files. */
char    cameraPath[100];       
/** Option, filename of the world to use. */
char    worldFile[100];        
/** Option, camera file for small round obstacles. */
char    smallCamera[100];      
/** Option, camera file for round obstacles. */
char    bigCamera[100];        
/** Option, camera file for robot. */
char    robotCamera[100];      
/** Option, camera file for walls. */
char    wallCamera[100];       
/** Option, camera file for light sources. */
char    lightCamera[100];      
/** Option, camera file for robot movements. */
char    motorCamera[100];      
/** Option, filename for the file to save individ weights to. */
char    weightsFile[100];      
/** Option, filename for the file to write fitness log to. */
char    fitnessFile[100];      
/** Option, save activations? **/
int     saveActivations = 0;
/** Option, number of hidden neurons **/
int     hiddenNeurons = 0;

/* Network input values */
/** Global, mapping of \a input arrar to ann [NR_OF_ROBOTS][NETWORK_INPUTS]. */
int     **mapping;  
/** Global, array where sensor values for the robots are stored [NR_OF_ROBOTS][NETWORK_INPUTS]. */
double  **input;    
/** Global, copy of input [NR_OF_ROBOTS][NETWORK_INPUTS]. */
double  **simoutput;

int userMode = 0;


FILE *fp;
char **lineBuffer;
int nrOfLines=0;
int readLines=0;



char* readLine(){
  char ch[120];
  if(fgets(ch,120,fp)==NULL)
    return(NULL);
  else
    return(g_strdup(ch));
}

void readFile(){
  int i;
  lineBuffer = (char**) g_malloc(sizeof(char*)*CONFIG_FILE_LINES);
  for(i=0; i < CONFIG_FILE_LINES && (lineBuffer[i]=readLine())!=NULL; i++);
  nrOfLines=i;
  printf("Read %3d config lines\n",nrOfLines);
  readLines=nrOfLines;
}

void freeBuffers(){
  int i;
  for(i=0; i<nrOfLines; i++){
    g_free(lineBuffer[i]);
  }
  g_free(lineBuffer);
}

int lineCompare(const void *str1, const void *str2){
  return g_strcasecmp(*((char**)str1),*((char**)str2));
}

void sortFile(){
  qsort(lineBuffer,nrOfLines,sizeof(char*),lineCompare);
}

void printFile(){
  int i;
  printf("\nLinenumber   String\n");
  for(i=0; i < nrOfLines; i++){
    printf("%4d -%s\n",i,lineBuffer[i]);
  }
}

void removeMojs(){
  int i,q;
  for(i=0; i < nrOfLines; i++){
    lineBuffer[i]=g_strchug(lineBuffer[i]);
    lineBuffer[i]=g_strchomp(lineBuffer[i]);    
  }
  for(i=0; i < nrOfLines; i++){
    if(lineBuffer[i][0]=='#' || lineBuffer[i][0]=='\n' || \
       lineBuffer[i][0]=='\0' || lineBuffer[i][0]==' '){
      g_free(lineBuffer[i]);
      for(q=i;q < nrOfLines-1; q++){
	lineBuffer[q]=lineBuffer[q+1];
      }
      nrOfLines--;
      i--; /* We have to test the new string on place i */
    }
  }
}

char *removeToken(char **str){
  char *tmp;
  int i;
  char strToken[40];
  sscanf(*str,"%s %d",strToken,&i);
  tmp = *str+strlen(strToken);
  strcpy(*str,tmp);
  return g_strdup(strToken);
}

void parseGetNext(int *i,char **findThis){
  (*i)++;
  *findThis = removeToken(&lineBuffer[*i]);
}

void parseFile(){
  int i;
  int baudCount=0;
  char *findThis;
  robotStr = NULL;
  findThis=NULL;
  i=0;
  findThis = removeToken(&lineBuffer[i]);
  /* They must lay in alphabetic order */
  if(strcmp("A_#ROBOTS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&paramNrOfRobots);
    robotStr = (char**) g_malloc(sizeof(char*)*paramNrOfRobots);
    for(int x=0; x < paramNrOfRobots; x++)
      robotStr[x]=NULL;
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  baudRate = (int*) g_malloc(sizeof(int)*paramNrOfRobots);
  while(strcmp("BAUD_RATE",findThis)==0 && baudCount < paramNrOfRobots){
      sscanf(lineBuffer[i],"%d",&baudRate[baudCount++]);
      g_free(findThis);
      parseGetNext(&i,&findThis);
  }
  if(strcmp("BIG_CAMERA",findThis)==0){
    sscanf(lineBuffer[i],"%s",bigCamera);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("BIT_MUTATION",findThis)==0){
    sscanf(lineBuffer[i],"%lf",&bitMutateProbability);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("CAMERA_PATH",findThis)==0){
    sscanf(lineBuffer[i],"%s",cameraPath);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("EPOCHS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&amoebaEpochs);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("FITNESS_FILE",findThis)==0){
    sscanf(lineBuffer[i],"%s",fitnessFile);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("FITNESS_FUNCTION",findThis)==0){
    sscanf(lineBuffer[i],"%d",&ffitness);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("GENERATIONS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&nrOfGenerations);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("GRIPPER_NO_NOISE",findThis)==0){
    sscanf(lineBuffer[i],"%d",&gripperNoise);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("HIDDEN_NEURONS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&hiddenNeurons);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("INDIVIDS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&nrOfIndivids);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("LIGHT_CAMERA",findThis)==0){
    sscanf(lineBuffer[i],"%s",lightCamera);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("LOG_PATH",findThis)==0){
    sscanf(lineBuffer[i],"%s",logPath);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("MOTOR_CAMERA",findThis)==0){
    sscanf(lineBuffer[i],"%s",motorCamera);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("MOVE_OBSTACLES",findThis)==0){
    sscanf(lineBuffer[i],"%d",&moveObstacles);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("NOISE_PERCENTAGE",findThis)==0){
    sscanf(lineBuffer[i],"%f",&amountOfNoise);
    g_free(findThis);
    if(amountOfNoise!=0)
      amountOfNoise = amountOfNoise / 100.0;
    parseGetNext(&i,&findThis);
  }
  if(strcmp("NR_OF_INDIVIDS_TO_LOG",findThis)==0){
    sscanf(lineBuffer[i],"%d",&nrOfIndividsToLog);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("NR_OF_STEPS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&nrOfSteps);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("OFFSPRINGS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&nrOfKids);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("PARENTS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&nrOfFathers);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("PICTURE_ROBOT",findThis)==0){
    sscanf(lineBuffer[i],"%s",robotPicFile);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("PICTURE_WORLD",findThis)==0){
    sscanf(lineBuffer[i],"%s",worldPicFile);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("RADIO_TURRETS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&useRadioTurrets);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("ROBOT_CAMERA",findThis)==0){
    sscanf(lineBuffer[i],"%s",robotCamera);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  int c=0;
  while(g_strncasecmp("ROBOTD",findThis,6)==0){
    if(paramNrOfRobots == 0 ){
      printf("No robots specified !!! \n\n");
      exit(EXIT_FAILURE);
    }
    robotStr[c] = g_strdup(lineBuffer[i]);
    if(c < paramNrOfRobots)
      c++;
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("ROD_NOISE",findThis)==0){
    sscanf(lineBuffer[i],"%d",&useRodNoise);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("RUN_REAL",findThis)==0){
    sscanf(lineBuffer[i],"%d",&useSerialLine);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("SAVE_ACTIVATIONS",findThis)==0){
    sscanf(lineBuffer[i],"%d",&saveActivations);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("SAVE_WEIGHTS_EVERY_x_GENERATION",findThis)==0){
    sscanf(lineBuffer[i],"%d",&saveGeneration);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("SCALE",findThis)==0){
    sscanf(lineBuffer[i],"%f",&xScale);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("SELECTION_METHOD",findThis)==0){
    sscanf(lineBuffer[i],"%d",&selectionMethod);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("SENSOR_NOISE",findThis)==0){
    sscanf(lineBuffer[i],"%d",&sensorNoise);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  baudCount=0;
  serialDevice = (char**) g_malloc(sizeof(char*)*paramNrOfRobots);
  while(strcmp("SERIAL_LINE",findThis)==0 && baudCount < paramNrOfRobots){
      serialDevice[baudCount] = (char*) g_malloc(sizeof(char) * 20);
      sscanf(lineBuffer[i],"%s",serialDevice[baudCount++]);
      printf("Serial line 0 is %s \n", serialDevice[0]);
      g_free(findThis);
      parseGetNext(&i,&findThis);
  } 
  if(strcmp("SMALL_CAMERA",findThis)==0){
    sscanf(lineBuffer[i],"%s",smallCamera);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("START_GENERATION",findThis)==0){
    sscanf(lineBuffer[i],"%d",&startGeneration);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("TEST_SAME_START_POSITION",findThis)==0){
    sscanf(lineBuffer[i],"%d",&testSameStartPosition);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("UPDATE_DELAY",findThis)==0){
    sscanf(lineBuffer[i],"%d",&updateDelay);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("VERBOSE_LEVEL",findThis)==0){
    sscanf(lineBuffer[i],"%d",&verboseLevel);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("WAIT_ON_START",findThis)==0){
    sscanf(lineBuffer[i],"%d",&waitOnStart);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("WALL_CAMERA",findThis)==0){
    sscanf(lineBuffer[i],"%s",wallCamera);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("WEIGHT_FILE",findThis)==0){
    sscanf(lineBuffer[i],"%s",weightsFile);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("WORLD_FILE",findThis)==0){
    sscanf(lineBuffer[i],"%s",worldFile);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(strcmp("WORLD_PATH",findThis)==0){
    sscanf(lineBuffer[i],"%s",worldPath);
    g_free(findThis);
    parseGetNext(&i,&findThis);
  }
  if(findThis)
    g_free(findThis);
  mapping = (int**)g_malloc(sizeof(int*)*paramNrOfRobots);
  input   = (double**)g_malloc(sizeof(double*)*paramNrOfRobots);         
  simoutput = (double**)g_malloc(sizeof(double*)*paramNrOfRobots);
  for( i=0; i < paramNrOfRobots; i++){
    input[i]   = (double*)g_malloc(sizeof(double)*(NETWORK_INPUTS));
    simoutput[i] = (double*)g_malloc(sizeof(double)*(NETWORK_INPUTS));
    mapping[i] = (int*)g_malloc(sizeof(int)*(NETWORK_INPUTS));
  }


}

void printParams(){
  if(verboseLevel>1){
    printf("------Simulator options-----\n");
    if(sensorNoise==1){
      printf("Using sensor noise\n");
      printf("amount of sensor noise %f\n",amountOfNoise);
    }
    
    printf("Graphical scale %f\n",xScale);
    
    printf("\nNr of generations to run:   %d\n",nrOfGenerations);
    printf("Nr of individs per generation %d\n",nrOfIndivids);
    printf("Nr of fathers  per generation %d\n",nrOfFathers);
    printf("Nr of kids     per father     %d\n",nrOfKids);
    if (startGeneration > 0)
      printf("Starting generation             : %d\n",startGeneration);
    printf("Epochs,steps,timelaps          : %d %d\n",amoebaEpochs,nrOfSteps);
    printf("Generations,best,offspring      : %d %d %d\n",
	   nrOfGenerations,nrOfFathers,nrOfKids);
    printf("Logging the best %d individs each generation\n",nrOfIndividsToLog);
    printf("Random seed : %d\n", seed);
    printf("Fitness function : %d\n",ffitness);
  }
}
/**
 * Loads option file, parse it and set global variables and options.
 * @param paramFile Filename of the option file to load.
 */
void loadParams(char *paramFile){
  if((fp=fopen(paramFile,"r"))==NULL){
    fprintf(stderr,"Couldnt open optionsfile %s \n",paramFile);
    exit(-1);
  }
  readFile();
  sortFile();  
  removeMojs();
  printf("Kept %3d config lines\n",readLines-nrOfLines);
  parseFile();
  printParams();
  // Cean up memory
  freeBuffers();
}
