/*
  Optann, a Khepera simulator including a separate GA and ANN 
  (Genetic Algoritm, Artificial Neural Net).
  Copyright (C) 2000  Johan Carlsson (johanc@ida.his.se)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#ifndef PARAM_H
#define PARAM_H
#include <stdio.h>
#include <stdlib.h>

#ifdef WIN32
#ifndef M_PI
#define M_PI 3.14159265358979
#endif
#endif

/* Defines */
/** Resolution of the rod camera in pixels */
#define ROD_RESOLUTION 10
/** Camera mmphfl.. *cough* */
#define ROD_ANGLE 36
/** Range of rod camera */
#define ROD_MAX_DISTANCE 170

/** position of energy sensor in input array */
#define EnergySensor 9
/** position of gripper sensor in input array */
#define GripperSensor 10 
/** position of ground sensor in input array */
#define GroundSensor 11
/** position of compass sensor in input array */
#define CompassSensor 12
/** start position of light bulb sensors in input array */
#define LightBulbSensor 13
/** start position of rod camera in input array */
#define RodSensor (LightBulbSensor + 8)
/** the size of the input array */
#define NETWORK_INPUTS (RodSensor + ROD_RESOLUTION)
/*
  bot code 
  g = gripper
  b = back infrared
  f? = 2,4,6 front infrared
  h = hand, gripper sensor
  z = ground sensor
  c = compass sensor
  e = energy sensor
  r = rod sensor
  l = light sensor
*/

/** Robot type flag */
#define BOT_2_INFRARED (1 << 0)
/** Robot type flag */
#define BOT_4_INFRARED (1 << 1)
/** Robot type flag */
#define BOT_6_INFRARED (1 << 2)
/** Robot type flag */
#define BOT_BACK_INFRARED (1 << 3)
/** Robot type flag */
#define BOT_COMPASS_SENSOR (1 << 4)
/** Robot type flag */
#define BOT_ENERGY_SENSOR (1 << 5)
/** Robot type flag */
#define BOT_GRIPPER (1 << 6)
/** Robot type flag */
#define BOT_GRIPPER_SENSOR (1 << 7)
/** Robot type flag */
#define BOT_GROUND_SENSOR (1 << 8)
/** Robot type flag */
#define BOT_LIGHT_SENSOR (1 << 9)
/** Robot type flag */
#define BOT_ROD_SENSOR (1 << 10)
/** Robot type flag */
#define BOT_HAS_ROD (1 << 11)
/** Failure occured during parsing of robot description. */
#define BOT_FAILURE 0
/** Robot was coded decimal, hexadecimal or octal. */
#define BOT_DECIMAL_CODED 1
/** Robot was coded binary. */
#define BOT_BINARY_CODED 2
/** Robot was coded by a text string. */
#define BOT_TEXT_CODED 3
/** Clear a bit in a variable. */
#define clear(var, mask)  var &= (~(mask))
/** Set a bit in a variable. */
#define set(var, mask)	  var |= (mask)
/** Test if a bit is set in a variable */
#define test(var,mask)    ((var & mask) & mask)

struct RobotDescription{
  int flags;
  int decodeMethod;
  int frontInfrared;
  int backInfrared;
  int useCompass;
  int useEnergy;
  int useRodSensor;
  int useGround;
  int useLight;
  int useGripper;
  int useGripperSensor;
  int hasRod;
  int rods;
  int nrOfSensors;
  float *rod_coords;
};

/* Serial thingis */
extern int     useRadioTurrets;
extern int     useSerialLine;
extern char    **serialDevice;
extern int     *baudRate;

/*  parameters  */
extern int     updateDelay;
extern int     gfxUpdate;
extern int     userUpdate;
extern int     useRodNoise;
extern float   xScale;
extern int     notTrajactoryRobot;
extern int     selectionMethod;
extern double  bitMutateProbability;
extern int     seed;               
extern int     amoebaEpochs;       
extern int     nrOfGenerations;    
extern int     nrOfIndivids;
extern int     nrOfKids;           
extern int     nrOfFathers;       
extern int     startGeneration;  
extern int     verboseLevel;     
extern int     sensorNoise;      
extern int     paramNrOfRobots;
extern float   amountOfNoise;
extern int     testSameStartPosition;
extern int     ffitness;
extern char    **robotStr;
extern int     moveObstacles;
extern int     gripperNoise;
extern int     nrOfSteps;    
extern int     saveGeneration;
extern int     nrOfIndividsToLog;
extern char    robotPicFile[100];
extern char    worldPicFile[100];
extern char    logPath[100];
extern char    worldPath[100];
extern char    cameraPath[100];
extern char    worldFile[100];
extern char    smallCamera[100];
extern char    bigCamera[100];
extern char    wallCamera[100];
extern char    lightCamera[100];
extern char    motorCamera[100];
extern char    weightsFile[100];
extern char    fitnessFile[100];
extern char    robotCamera[100];
extern int     **mapping;
extern double  **input;
extern double  **simoutput;
extern int     saveActivations;
extern int     hiddenNeurons;
extern int     userMode;

extern int waitOnStart;

/** Number of lines that the param parser can handle */
#define CONFIG_FILE_LINES 1000

void loadParams(char *paramFile);


#endif /* PARAM_H */
