/*
  YAKS, a Khepera simulator including a separate GA and ANN 
  (Genetic Algoritm, Artificial Neural Net).
  Copyright (C) 2000  Johan Carlsson (johanc@ida.his.se)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#ifndef SOBST_H
#define SOBST_H
#include "geom.h"
#include <stdio.h>
#include "param.h"

/**
 * @class Sobst sobst.h
 * @brief This class implements small round obstacles that can be moved.
 *
 * Note that all small round obstacles look the same. So they all share
 * the same radius and lookup table (declared static).
 */

class Sobst{
 public:
  Sobst(FILE *fp);
  Sobst(char *str);
  ~Sobst();
  void init(char *fileName);
  void save(char *fileName);
  void setRadius(float rad);
  float *sensorVal(float kx, float ky, float kangle, float robRadius);
  void saveMove();
  void tempMove(double dist, double angR);
  void resetMove();
  /** Obstacle \a x coordinate */ 
  float x;
  /** Obstacle \a y coordinate */ 
  float y;
  /** Shared, small round obstacles radius */
  static float r;
  /** Obstacle picked up by robot, 1.0 - True, 0.0 - False*/
  float taken;
 private:
  /** Saved \a x and \a y coordinate  */
  float orgX,orgY;
  int relangle(float kx, float ky, float kangle);
  /** Shared lookup table for sensor readings */
  static float sobst[20][180][8];
  /** Shared obstacle range for sensors */
  static float MaxObstacleDistans;
  /** Shared distance from which obstacles gives maximum sensor values */
  static float MinObstacleDistans;
  /** Shared step size, resolution for sensor values */
  static float Step;
};


#endif /* SOBST_H */
